# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django import template
from django import shortcuts
from django.contrib import messages

from django_openstack import api
from django_openstack import forms
from openstackx.api import exceptions as api_exceptions


LOG = logging.getLogger('django_openstack.auth')


class Login(forms.SelfHandlingForm):
    username = forms.CharField(max_length="20", label="User Name")
    password = forms.CharField(max_length="20", label="Password")

    def handle(self, request, data):
        try:
            token = api.token_create(request,
                                     "",
                                     data['username'],
                                     data['password'])
            info = api.token_info(request, token)
            request.session['token'] = token.id
            request.session['user'] = info['user']
            request.session['tenant'] = info['tenant']
            request.session['admin'] = info['admin']
            request.session['serviceCatalog'] = token.serviceCatalog
            LOG.info('Login form for user "%s". Service Catalog data:\n%s' %
                     (data['username'], token.serviceCatalog))

            if request.session['admin']:
                return shortcuts.redirect('syspanel_overview')
            else:
                return shortcuts.redirect('dash_overview')

        except api_exceptions.Unauthorized as e:
            msg = 'Error authenticating: %s' % e.message
            LOG.error(msg, exc_info=True)
            messages.error(request, msg)


def login(request):
    if request.user and request.user.is_authenticated():
        if request.user.is_admin():
            return shortcuts.redirect('syspanel_overview')
        else:
            return shortcuts.redirect('dash_overview')

    form, handled = Login.maybe_handle(request)
    if handled:
        return handled

    return shortcuts.render_to_response('login_required.html', {
        'form': form,
    }, context_instance=template.RequestContext(request))


def switch_tenants(request, tenant_id):
    form, handled = Login.maybe_handle(
            request, initial={'tenant': tenant_id})
    if handled:
        return handled

    return shortcuts.render_to_response('switch_tenants.html', {
        'to_tenant': tenant_id,
        'form': form,
    }, context_instance=template.RequestContext(request))


def logout(request):
    request.session.clear()
    return shortcuts.redirect('splash')



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

ROOT_PATH = os.path.dirname(os.path.abspath(__file__))
DEBUG = True
TESTSERVER = 'http://testserver'
DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': '/tmp/django-openstack.db',
            },
        }
INSTALLED_APPS = ['django.contrib.auth',
                  'django.contrib.contenttypes',
                  'django.contrib.sessions',
                  'django.contrib.sites',
                  'django_openstack',
                  'django_openstack.tests',
                  'django_openstack.templatetags',
                  'mailer',
                  ]

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django_openstack.middleware.keystone.AuthenticationMiddleware',
    )

ROOT_URLCONF = 'django_openstack.tests.testurls'
TEMPLATE_DIRS = (
    os.path.join(ROOT_PATH, 'tests', 'templates')
)
SITE_ID = 1
SITE_BRANDING = 'OpenStack'
SITE_NAME = 'openstack'
ENABLE_VNC = True
NOVA_DEFAULT_ENDPOINT = None
NOVA_DEFAULT_REGION = 'test'
NOVA_ACCESS_KEY = 'test'
NOVA_SECRET_KEY = 'test'
OPENSTACK_ADMIN_TOKEN = 'test'

CREDENTIAL_AUTHORIZATION_DAYS = 2
CREDENTIAL_DOWNLOAD_URL = TESTSERVER + '/credentials/'

TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'
NOSE_ARGS = ['--nocapture',
             '--cover-package=django_openstack',
             '--cover-inclusive',
            ]

# django-mailer uses a different config attribute
# even though it just wraps django.core.mail
MAILER_EMAIL_BACKEND = 'django.core.mail.backends.locmem.EmailBackend'
EMAIL_BACKEND = MAILER_EMAIL_BACKEND

SWIFT_ACCOUNT = 'test'
SWIFT_USER = 'tester'
SWIFT_PASS = 'testing'
SWIFT_AUTHURL = 'http://swift/swiftapi/v1.0'



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime


def time():
    '''Overrideable version of datetime.datetime.today'''
    if time.override_time:
        return time.override_time
    return datetime.time()

time.override_time = None


def today():
    '''Overridable version of datetime.datetime.today'''
    if today.override_time:
        return today.override_time
    return datetime.datetime.today()

today.override_time = None


def utcnow():
    '''Overridable version of datetime.datetime.utcnow'''
    if utcnow.override_time:
        return utcnow.override_time
    return datetime.datetime.utcnow()

utcnow.override_time = None



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Views for managing Swift containers.
"""
import logging

from django import http
from django import template
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django import shortcuts
from django.shortcuts import render_to_response

from django_openstack import api
from django_openstack import forms


LOG = logging.getLogger('django_openstack.dash')


class DeleteObject(forms.SelfHandlingForm):
    object_name = forms.CharField(widget=forms.HiddenInput())

    def handle(self, request, data):
        api.swift_delete_object(
                request.POST['container_name'],
                data['object_name'])
        messages.info(request,
                      'Successfully deleted object: %s' % \
                      data['object_name'])
        return shortcuts.redirect(request.build_absolute_uri())


class UploadObject(forms.SelfHandlingForm):
    name = forms.CharField(max_length="255", label="Object Name")
    object_file = forms.FileField(label="File")

    def handle(self, request, data):
        api.swift_upload_object(
                request.POST['container_name'],
                data['name'],
                request.FILES['object_file'].read())

        messages.success(request, "Object was successfully uploaded.")
        return shortcuts.redirect(request.build_absolute_uri())


class CopyObject(forms.SelfHandlingForm):
    new_container_name = forms.ChoiceField(
        label="Container to store object in")

    new_object_name = forms.CharField(max_length="255",
                                      label="New object name")

    def __init__(self, *args, **kwargs):
        super(CopyObject, self).__init__(*args, **kwargs)

        container_choices = [(c.name, c.name) for c in api.swift_get_containers()]
        self.fields['new_container_name'].choices = container_choices

    def handle(self, request, data):
        orig_container_name = request.POST['orig_container_name']
        orig_object_name = request.POST['orig_object_name']
        new_container_name = request.POST['new_container_name']
        new_object_name = data['new_object_name']

        api.swift_copy_object(orig_container_name, orig_object_name,
                              new_container_name, new_object_name)

        messages.success(request,
                         'Object was successfully copied to %s\%s' %
                         (new_container_name, new_object_name))

        return shortcuts.redirect(request.build_absolute_uri())


@login_required
def index(request, tenant_id, container_name):
    delete_form, handled = DeleteObject.maybe_handle(request)
    if handled:
        return handled

    objects = api.swift_get_objects(container_name)

    return render_to_response('dash_objects.html', {
        'container_name': container_name,
        'objects': objects,
        'delete_form': delete_form,
    }, context_instance=template.RequestContext(request))


@login_required
def upload(request, tenant_id, container_name):
    form, handled = UploadObject.maybe_handle(request)
    if handled:
        return handled

    return render_to_response('dash_objects_upload.html', {
        'container_name': container_name,
        'upload_form': form,
    }, context_instance=template.RequestContext(request))


@login_required
def download(request, tenant_id, container_name, object_name):
    object_data = api.swift_get_object_data(
            container_name, object_name)

    response = http.HttpResponse()
    response['Content-Disposition'] = 'attachment; filename=%s' % \
            object_name
    for data in object_data:
        response.write(data)
    return response


@login_required
def copy(request, tenant_id, container_name, object_name):
    form, handled = CopyObject.maybe_handle(request)

    form.fields['new_container_name'].initial = container_name

    if handled:
        return handled

    return render_to_response(
        'dash_object_copy.html',
        {'container_name': container_name,
         'object_name': object_name,
         'copy_form': form},
        context_instance=template.RequestContext(request))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Unit tests for volume views.
"""

import boto.ec2.volume
import mox

from django.core.urlresolvers import reverse
from django_openstack.nova import forms
from django_openstack.nova.tests.base import (BaseProjectViewTests,
                                              TEST_PROJECT)


TEST_VOLUME = 'vol-0000001'


class VolumeTests(BaseProjectViewTests):
    def test_index(self):
        instance_id = 'i-abcdefgh'

        volume = boto.ec2.volume.Volume()
        volume.id = TEST_VOLUME
        volume.displayName = TEST_VOLUME
        volume.size = 1

        self.mox.StubOutWithMock(self.project, 'get_volumes')
        self.mox.StubOutWithMock(forms, 'get_available_volume_choices')
        self.mox.StubOutWithMock(forms, 'get_instance_choices')
        self.project.get_volumes().AndReturn([])
        forms.get_available_volume_choices(mox.IgnoreArg()).AndReturn(
                self.create_available_volume_choices([volume]))
        forms.get_instance_choices(mox.IgnoreArg()).AndReturn(
                self.create_instance_choices([instance_id]))

        self.mox.ReplayAll()

        response = self.client.get(reverse('nova_volumes',
                                           args=[TEST_PROJECT]))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response,
                'django_openstack/nova/volumes/index.html')
        self.assertEqual(len(response.context['volumes']), 0)

        self.mox.VerifyAll()

    def test_add_get(self):
        self.mox.ReplayAll()

        res = self.client.get(reverse('nova_volumes_add', args=[TEST_PROJECT]))
        self.assertRedirectsNoFollow(res, reverse('nova_volumes',
                                                  args=[TEST_PROJECT]))
        self.mox.VerifyAll()

    def test_add_post(self):
        vol = boto.ec2.volume.Volume()
        vol.name = TEST_VOLUME
        vol.displayName = TEST_VOLUME
        vol.size = 1

        self.mox.StubOutWithMock(self.project, 'create_volume')
        self.project.create_volume(vol.size, vol.name, vol.name).AndReturn(vol)

        self.mox.ReplayAll()

        url = reverse('nova_volumes_add', args=[TEST_PROJECT])
        data = {'size': '1',
                'nickname': TEST_VOLUME,
                'description': TEST_VOLUME}
        res = self.client.post(url, data)
        self.assertRedirectsNoFollow(res, reverse('nova_volumes',
                                                  args=[TEST_PROJECT]))
        self.mox.VerifyAll()

    def test_delete_get(self):
        self.mox.ReplayAll()

        res = self.client.get(reverse('nova_volumes_delete',
                                      args=[TEST_PROJECT, TEST_VOLUME]))
        self.assertRedirectsNoFollow(res, reverse('nova_volumes',
                                                  args=[TEST_PROJECT]))
        self.mox.VerifyAll()

    def test_delete_post(self):
        self.mox.StubOutWithMock(self.project, 'delete_volume')
        self.project.delete_volume(TEST_VOLUME).AndReturn(True)

        self.mox.ReplayAll()

        res = self.client.post(reverse('nova_volumes_delete',
                                       args=[TEST_PROJECT, TEST_VOLUME]))
        self.assertRedirectsNoFollow(res, reverse('nova_volumes',
                                                  args=[TEST_PROJECT]))
        self.mox.VerifyAll()

    def test_attach_get(self):
        self.mox.ReplayAll()

        res = self.client.get(reverse('nova_volumes_attach',
                                      args=[TEST_PROJECT]))
        self.assertRedirectsNoFollow(res, reverse('nova_volumes',
                                                  args=[TEST_PROJECT]))
        self.mox.VerifyAll()

    def test_attach_post(self):
        volume = boto.ec2.volume.Volume()
        volume.id = TEST_VOLUME
        volume.displayName = TEST_VOLUME
        volume.size = 1

        instance_id = 'i-abcdefgh'
        device = '/dev/vdb'

        self.mox.StubOutWithMock(self.project, 'attach_volume')
        self.mox.StubOutWithMock(forms, 'get_available_volume_choices')
        self.mox.StubOutWithMock(forms, 'get_instance_choices')
        self.project.attach_volume(TEST_VOLUME, instance_id, device) \
                    .AndReturn(True)
        forms.get_available_volume_choices(mox.IgnoreArg()).AndReturn(
                self.create_available_volume_choices([volume]))
        forms.get_instance_choices(mox.IgnoreArg()).AndReturn(
                self.create_instance_choices([instance_id]))

        self.mox.ReplayAll()

        url = reverse('nova_volumes_attach', args=[TEST_PROJECT])
        data = {'volume': TEST_VOLUME,
                'instance': instance_id,
                'device': device}
        res = self.client.post(url, data)
        self.assertRedirectsNoFollow(res, reverse('nova_volumes',
                                                  args=[TEST_PROJECT]))
        self.mox.VerifyAll()

    def test_detach_get(self):
        self.mox.ReplayAll()

        res = self.client.get(reverse('nova_volumes_detach',
                                      args=[TEST_PROJECT, TEST_VOLUME]))
        self.assertRedirectsNoFollow(res, reverse('nova_volumes',
                                                  args=[TEST_PROJECT]))
        self.mox.VerifyAll()

    def test_detach_post(self):
        self.mox.StubOutWithMock(self.project, 'detach_volume')
        self.project.detach_volume(TEST_VOLUME).AndReturn(True)

        self.mox.ReplayAll()

        res = self.client.post(reverse('nova_volumes_detach',
                                       args=[TEST_PROJECT, TEST_VOLUME]))
        self.assertRedirectsNoFollow(res, reverse('nova_volumes',
                                                  args=[TEST_PROJECT]))
        self.mox.VerifyAll()



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django import template
from django import http
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.shortcuts import render_to_response
from django.shortcuts import redirect
from django.utils.translation import ugettext as _

import datetime
import json
import logging
import subprocess
import sys
import urlparse

from django.contrib import messages

from django_openstack import api
from django_openstack import forms
from django_openstack.dash.views import instances as dash_instances
from openstackx.api import exceptions as api_exceptions

LOG = logging.getLogger('django_openstack.syspanel.views.services')


class ToggleService(forms.SelfHandlingForm):
    service = forms.CharField(required=False)
    name = forms.CharField(required=False)

    def handle(self, request, data):
        try:
            service = api.service_get(request, data['service'])
            api.service_update(request,
                               data['service'],
                               not service.disabled)
            if service.disabled:
                messages.info(request, "Service '%s' has been enabled"
                                        % data['name'])
            else:
                messages.info(request, "Service '%s' has been disabled"
                                        % data['name'])
        except api_exceptions.ApiException, e:
            LOG.error('ApiException while toggling service %s' %
                      data['service'], exc_info=True)
            messages.error(request, "Unable to update service '%s': %s"
                                     % data['name'], e.message)

        return redirect(request.build_absolute_uri())


@login_required
def index(request):
    for f in (ToggleService,):
        _, handled = f.maybe_handle(request)
        if handled:
            return handled

    services = []
    try:
        services = api.service_list(request)
    except api_exceptions.ApiException, e:
        LOG.error('ApiException fetching service list', exc_info=True)
        messages.error(request, 'Unable to get service info: %s' % e.message)

    other_services = []

    for k, v in request.session['serviceCatalog'].iteritems():
        v = v[0]
        try:
            # TODO(mgius): This silences curl, but there's probably
            # a better solution than using curl to begin with
            subprocess.check_call(['curl', '-m', '1', v['internalURL']],
                                  stdout=open(sys.devnull, 'w'),
                                  stderr=open(sys.devnull, 'w'))
            up = True
        except:
            up = False
        hostname = urlparse.urlparse(v['internalURL']).hostname
        row = {'type': k, 'internalURL': v['internalURL'], 'host': hostname,
               'region': v['region'], 'up': up }
        other_services.append(row)

    return render_to_response('syspanel_services.html', {
        'services': services,
        'service_toggle_enabled_form': ToggleService,
        'other_services': other_services,
    }, context_instance = template.RequestContext(request))



import datetime
import hashlib
import mox
import random

from django import test
from django.conf import settings
from django.db.models.signals import post_save
from django_openstack import models as nova_models
from django_openstack import utils
from django_openstack.core import connection
from nova_adminclient import NovaAdminClient


TEST_USER = 'testUser'
TEST_PROJECT = 'testProject'
TEST_AUTH_TOKEN = hashlib.sha1('').hexdigest()
TEST_AUTH_DATE = utils.utcnow()
TEST_BAD_AUTH_TOKEN = 'badToken'

HOUR = datetime.timedelta(seconds=3600)
AUTH_EXPIRATION_LENGTH = \
        datetime.timedelta(days=int(settings.CREDENTIAL_AUTHORIZATION_DAYS))


class CredentialsAuthorizationTests(test.TestCase):
    @classmethod
    def setUpClass(cls):
        # these post_save methods interact with external resources, shut them
        # down to test credentials
        post_save.disconnect(sender=nova_models.CredentialsAuthorization,
            dispatch_uid='django_openstack.CredentialsAuthorization.post_save')
        post_save.disconnect(sender=nova_models.CredentialsAuthorization,
                dispatch_uid='django_openstack.User.post_save')

    def setUp(self):
        test_cred = nova_models.CredentialsAuthorization()
        test_cred.username = TEST_USER
        test_cred.project = TEST_PROJECT
        test_cred.auth_date = TEST_AUTH_DATE
        test_cred.auth_token = TEST_AUTH_TOKEN
        test_cred.save()

        badTestCred = nova_models.CredentialsAuthorization()
        badTestCred.username = TEST_USER
        badTestCred.project = TEST_PROJECT
        badTestCred.auth_date = TEST_AUTH_DATE
        badTestCred.auth_token = TEST_BAD_AUTH_TOKEN
        badTestCred.save()

        self.mox = mox.Mox()

    def tearDown(self):
        self.mox.UnsetStubs()

    def test_get_by_token(self):
        TEST_MISSING_AUTH_TOKEN = hashlib.sha1('notAToken').hexdigest()

        # Token not a sha1, but exists in system
        cred = nova_models.CredentialsAuthorization.get_by_token(
                TEST_BAD_AUTH_TOKEN)
        self.assertTrue(cred is None)

        # Token doesn't exist
        cred = nova_models.CredentialsAuthorization.get_by_token(
                TEST_MISSING_AUTH_TOKEN)
        self.assertTrue(cred is None)

        # Good token
        cred = nova_models.CredentialsAuthorization.get_by_token(
                TEST_AUTH_TOKEN)
        self.assertTrue(cred is not None)

        # Expire the token
        cred.auth_date = utils.utcnow() - AUTH_EXPIRATION_LENGTH \
                                                 - HOUR
        cred.save()

        # Expired token
        cred = nova_models.CredentialsAuthorization.get_by_token(
                TEST_AUTH_TOKEN)
        self.assertTrue(cred is None)

    def test_authorize(self):
        TEST_USER2 = TEST_USER + '2'
        TEST_AUTH_TOKEN_2 = hashlib.sha1('token2').hexdigest()

        cred_class = nova_models.CredentialsAuthorization
        self.mox.StubOutWithMock(cred_class, 'create_auth_token')
        cred_class.create_auth_token(TEST_USER2).AndReturn(
                TEST_AUTH_TOKEN_2)

        self.mox.ReplayAll()

        cred = cred_class.authorize(TEST_USER2, TEST_PROJECT)

        self.mox.VerifyAll()

        self.assertTrue(cred is not None)
        self.assertEqual(cred.username, TEST_USER2)
        self.assertEqual(cred.project, TEST_PROJECT)
        self.assertEqual(cred.auth_token, TEST_AUTH_TOKEN_2)
        self.assertFalse(cred.auth_token_expired())

        cred = cred_class.get_by_token(TEST_AUTH_TOKEN_2)
        self.assertTrue(cred is not None)

    def test_create_auth_token(self):
        rand_state = random.getstate()
        expected_salt = hashlib.sha1(str(random.random())).hexdigest()[:5]
        expected_token = hashlib.sha1(expected_salt + TEST_USER).hexdigest()

        random.setstate(rand_state)
        auth_token = \
            nova_models.CredentialsAuthorization.create_auth_token(TEST_USER)
        self.assertEqual(expected_token, auth_token)

    def test_auth_token_expired(self):
        '''
        Test expired in past, expires in future, expires _right now_
        '''
        cred = \
            nova_models.CredentialsAuthorization.get_by_token(TEST_AUTH_TOKEN)

        cred.auth_date = utils.utcnow() - AUTH_EXPIRATION_LENGTH \
                                                 - HOUR
        self.assertTrue(cred.auth_token_expired())

        cred.auth_date = utils.utcnow()

        self.assertFalse(cred.auth_token_expired())

        # testing with time is tricky. Mock out "right now" test to avoid
        # timing issues
        time = utils.utcnow.override_time = utils.utcnow()
        cred.auth_date = time - AUTH_EXPIRATION_LENGTH

        self.assertTrue(cred.auth_token_expired())

        utils.utcnow.override_time = None

    def test_get_download_url(self):
        cred = \
            nova_models.CredentialsAuthorization.get_by_token(TEST_AUTH_TOKEN)

        expected_url = settings.CREDENTIAL_DOWNLOAD_URL + TEST_AUTH_TOKEN
        self.assertEqual(expected_url, cred.get_download_url())

    def test_get_zip(self):
        cred = \
            nova_models.CredentialsAuthorization.get_by_token(TEST_AUTH_TOKEN)

        admin_mock = self.mox.CreateMock(NovaAdminClient)

        self.mox.StubOutWithMock(connection, 'get_nova_admin_connection')
        connection.get_nova_admin_connection().AndReturn(admin_mock)

        admin_mock.get_zip(TEST_USER, TEST_PROJECT)

        self.mox.ReplayAll()

        cred.get_zip()

        self.mox.VerifyAll()

        cred = \
            nova_models.CredentialsAuthorization.get_by_token(TEST_AUTH_TOKEN)

        self.assertTrue(cred is None)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from django import template
from django.contrib import messages
from django.shortcuts import redirect
from django.shortcuts import render_to_response
from django.contrib.auth.decorators import login_required

from glance.common import exception as glance_exception

from django_openstack import api
from django_openstack import forms


LOG = logging.getLogger('django_openstack.sysadmin.views.images')


class DeleteImage(forms.SelfHandlingForm):
    image_id = forms.CharField(required=True)

    def handle(self, request, data):
        image_id = data['image_id']
        try:
            api.image_delete(request, image_id)
        except glance_exception.ClientConnectionError, e:
            LOG.error("Error connecting to glance", exc_info=True)
            messages.error(request,
                           "Error connecting to glance: %s" % e.message)
        except glance_exception.Error, e:
            LOG.error('Error deleting image with id "%s"' % image_id,
                      exc_info=True)
            messages.error(request, "Error deleting image: %s" % e.message)
        return redirect(request.build_absolute_uri())


class ToggleImage(forms.SelfHandlingForm):
    image_id = forms.CharField(required=True)

    def handle(self, request, data):
        image_id = data['image_id']
        try:
            api.image_update(request, image_id, image_meta={'is_public': False})
        except glance_exception.ClientConnectionError, e:
            LOG.error("Error connecting to glance", exc_info=True)
            messages.error(request,
                           "Error connecting to glance: %s" % e.message)
        except glance_exception.Error, e:
            LOG.error('Error updating image with id "%s"' % image_id,
                      exc_info=True)
            messages.error(request, "Error updating image: %s" % e.message)
        return redirect(request.build_absolute_uri())


@login_required
def index(request):
    for f in (DeleteImage, ToggleImage):
        _, handled = f.maybe_handle(request)
        if handled:
            return handled

    # We don't have any way of showing errors for these, so don't bother
    # trying to reuse the forms from above
    delete_form = DeleteImage()
    toggle_form = ToggleImage()

    images = []
    try:
        images = api.image_list_detailed(request)
        if not images:
            messages.info(request, "There are currently no images.")
    except glance_exception.ClientConnectionError, e:
        LOG.error("Error connecting to glance", exc_info=True)
        messages.error(request, "Error connecting to glance: %s" % e.message)
    except glance_exception.Error, e:
        LOG.error("Error retrieving image list", exc_info=True)
        messages.error(request, "Error retrieving image list: %s" % e.message)

    return render_to_response('syspanel_images.html', {
        'delete_form': delete_form,
        'toggle_form': toggle_form,
        'images': images,
    }, context_instance = template.RequestContext(request))


@login_required
def update(request, image_id):
    try:
        image = api.image_get(request, image_id)
    except glance_exception.ClientConnectionError, e:
        LOG.error("Error connecting to glance", exc_info=True)
        messages.error(request, "Error connecting to glance: %s" % e.message)
    except glance_exception.Error, e:
        LOG.error('Error retrieving image with id "%s"' % image_id,
                  exc_info=True)
        messages.error(request,
                       "Error retrieving image %s: %s" % (image_id, e.message))

    if request.method == "POST":
        form = UpdateImageForm(request.POST)
        if form.is_valid():
            image_form = form.clean()
            metadata = {
                'is_public': image_form['is_public'],
                'disk_format': image_form['disk_format'],
                'container_format': image_form['container_format'],
                'name': image_form['name'],
                'location': image_form['location'],
            }
            try:
                # TODO add public flag to properties
                metadata['properties'] = {
                    'kernel_id': int(image_form['kernel_id']),
                    'ramdisk_id': int(image_form['ramdisk_id']),
                    'image_state': image_form['state'],
                    'architecture': image_form['architecture'],
                    'project_id': image_form['project_id'],
                }
                api.image_update(request, image_id, metadata)
                messages.success(request, "Image was successfully updated.")
            except glance_exception.ClientConnectionError, e:
                LOG.error("Error connecting to glance", exc_info=True)
                messages.error(request,
                               "Error connecting to glance: %s" % e.message)
            except glance_exception.Error, e:
                LOG.error('Error updating image with id "%s"' % image_id,
                          exc_info=True)
                messages.error(request, "Error updating image: %s" % e.message)
            except:
                LOG.error('Unspecified Exception in image update',
                          exc_info=True)
                messages.error(request,
                               "Image could not be updated, please try again.")

        else:
            LOG.error('Image "%s" failed to update' % image['name'],
                      exc_info=True)
            messages.error(request,
                           "Image could not be uploaded, please try agian.")
            form = UpdateImageForm(request.POST)
            return render_to_response('django_nova_syspanel/images/image_update.html',{
                'image': image,
                'form': form,
            }, context_instance = template.RequestContext(request))

        return redirect('syspanel_images')
    else:
        form = UpdateImageForm(initial={
                'name': image.get('name', ''),
                'kernel': image['properties'].get('kernel_id', ''),
                'ramdisk': image['properties'].get('ramdisk_id', ''),
                'is_public': image.get('is_public', ''),
                'location': image.get('location', ''),
                'state': image['properties'].get('image_state', ''),
                'architecture': image['properties'].get('architecture', ''),
                'project_id': image['properties'].get('project_id', ''),
                'container_format': image.get('container_format', ''),
                'disk_format': image.get('disk_format', ''),
            })

        return render_to_response('django_nova_syspanel/images/image_update.html',{
            'image': image,
            'form': form,
        }, context_instance = template.RequestContext(request))


@login_required
def upload(request):
    if request.method == "POST":
        form = UploadImageForm(request.POST)
        if form.is_valid():
            image = form.clean()
            metadata = {'is_public': image['is_public'],
                        'disk_format': 'ami',
                        'container_format': 'ami',
                        'name': image['name']}
            try:
                messages.success(request, "Image was successfully uploaded.")
            except:
                # TODO add better error management
                messages.error(request, "Image could not be uploaded, please try again.")

            try:
                api.image_create(request, metadata, image['image_file'])
            except glance_exception.ClientConnectionError, e:
                LOG.error('Error connecting to glance while trying to upload'
                          ' image', exc_info=True)
                messages.error(request,
                               "Error connecting to glance: %s" % e.message)
            except glance_exception.Error, e:
                LOG.error('Glance exception while uploading image',
                          exc_info=True)
                messages.error(request, "Error adding image: %s" % e.message)
        else:
            LOG.error('Image "%s" failed to upload' % image['name'],
                      exc_info=True)
            messages.error(request,
                           "Image could not be uploaded, please try agian.")
            form = UploadImageForm(request.POST)
            return render_to_response('django_nova_syspanel/images/image_upload.html',{
                'form': form,
            }, context_instance = template.RequestContext(request))

        return redirect('syspanel_images')
    else:
        form = UploadImageForm()
        return render_to_response('django_nova_syspanel/images/image_upload.html',{
            'form': form,
        }, context_instance = template.RequestContext(request))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Unit tests for credential views.
"""

from django.conf import settings
from django.core.urlresolvers import reverse
from django_openstack import models
from django_openstack.nova.tests.base import BaseViewTests


class CredentialViewTests(BaseViewTests):
    def test_download_expired_credentials(self):
        auth_token = 'expired'
        self.mox.StubOutWithMock(models.CredentialsAuthorization,
                                 'get_by_token')
        models.CredentialsAuthorization.get_by_token(auth_token) \
                                       .AndReturn(None)
        self.mox.ReplayAll()

        res = self.client.get(reverse('nova_credentials_authorize',
                                      args=[auth_token]))
        self.assertTemplateUsed(res,
                'django_openstack/nova/credentials/expired.html')

        self.mox.VerifyAll()

    def test_download_good_credentials(self):
        auth_token = 'good'

        creds = models.CredentialsAuthorization()
        creds.username = 'test'
        creds.project = 'test'
        creds.auth_token = auth_token

        self.mox.StubOutWithMock(models.CredentialsAuthorization,
                                 'get_by_token')
        self.mox.StubOutWithMock(creds, 'get_zip')
        models.CredentialsAuthorization.get_by_token(auth_token) \
                                       .AndReturn(creds)
        creds.get_zip().AndReturn('zip')

        self.mox.ReplayAll()

        res = self.client.get(reverse('nova_credentials_authorize',
                                      args=[auth_token]))
        self.assertEqual(res.status_code, 200)
        self.assertEqual(res['Content-Disposition'],
                         'attachment; filename=%s-test-test-x509.zip' %
                         settings.SITE_NAME)
        self.assertContains(res, 'zip')

        self.mox.VerifyAll()



from cloudfiles.errors import ContainerNotEmpty
from django.contrib import messages
from django.core.urlresolvers import reverse
from django_openstack import api
from django_openstack.tests.view_tests import base
from mox import IgnoreArg, IsA


class ContainerViewTests(base.BaseViewTests):
    def setUp(self):
        super(ContainerViewTests, self).setUp()
        container_inner = base.Object()
        container_inner.name = 'containerName'
        self.container = api.Container(container_inner)

    def test_index(self):
        self.mox.StubOutWithMock(api, 'swift_get_containers')
        api.swift_get_containers().AndReturn([self.container])

        self.mox.ReplayAll()

        res = self.client.get(reverse('dash_containers', args=['tenant']))

        self.assertTemplateUsed(res, 'dash_containers.html')
        self.assertIn('containers', res.context)
        containers = res.context['containers']

        self.assertEqual(len(containers), 1)
        self.assertEqual(containers[0].name, 'containerName')

        self.mox.VerifyAll()

    def test_delete_container(self):
        formData = {'container_name': 'containerName',
                    'method': 'DeleteContainer'}

        self.mox.StubOutWithMock(api, 'swift_delete_container')
        api.swift_delete_container('containerName')

        self.mox.ReplayAll()

        res = self.client.post(reverse('dash_containers', args=['tenant']),
                               formData)

        self.assertRedirectsNoFollow(res, reverse('dash_containers',
                                                  args=['tenant']))

        self.mox.VerifyAll()

    def test_delete_container_nonempty(self):
        formData = {'container_name': 'containerName',
                          'method': 'DeleteContainer'}

        exception = ContainerNotEmpty('containerNotEmpty')

        self.mox.StubOutWithMock(api, 'swift_delete_container')
        api.swift_delete_container('containerName').AndRaise(exception)

        self.mox.StubOutWithMock(messages, 'error')

        messages.error(IgnoreArg(), IsA(unicode))

        self.mox.ReplayAll()

        res = self.client.post(reverse('dash_containers', args=['tenant']),
                               formData)

        self.assertRedirectsNoFollow(res, reverse('dash_containers',
                                          args=['tenant']))

        self.mox.VerifyAll()

    def test_create_container_get(self):
        res = self.client.get(reverse('dash_containers_create',
                              args=['tenant']))

        self.assertTemplateUsed(res, 'dash_containers_create.html')

    def test_create_container_post(self):
        formData = {'name': 'containerName',
                    'method': 'CreateContainer'}

        self.mox.StubOutWithMock(api, 'swift_create_container')
        api.swift_create_container('CreateContainer')

        self.mox.StubOutWithMock(messages, 'success')
        messages.success(IgnoreArg(), IsA(str))

        res = self.client.post(reverse('dash_containers_create',
                                       args=['tenant']),
                               formData)

        self.assertRedirectsNoFollow(res, reverse('dash_containers_create',
                                          args=['tenant']))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Views for managing Nova images.
"""

import datetime
import logging
import re

from django import http
from django import template
from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect, render_to_response
from django.utils.translation import ugettext as _
from django import shortcuts

from django_openstack import api
from django_openstack import forms
from openstackx.api import exceptions as api_exceptions
from glance.common import exception as glance_exception


LOG = logging.getLogger('django_openstack.dash.views.images')


class LaunchForm(forms.SelfHandlingForm):
    image_id = forms.CharField(widget=forms.HiddenInput())
    name = forms.CharField(max_length=80, label="Server Name")
    user_data = forms.CharField(widget=forms.Textarea,
                                label="User Data",
                                required=False)

    # make the dropdown populate when the form is loaded not when django is
    # started
    def __init__(self, *args, **kwargs):
        super(LaunchForm, self).__init__(*args, **kwargs)
        flavorlist = kwargs.get('initial', {}).get('flavorlist', [])
        self.fields['flavor'] = forms.ChoiceField(
                choices=flavorlist,
                label="Flavor",
                help_text="Size of Image to launch")

        keynamelist = kwargs.get('initial', {}).get('keynamelist', [])
        self.fields['key_name'] = forms.ChoiceField(choices=keynamelist,
                label="Key Name",
                help_text="Which keypair to use for authentication")

    def handle(self, request, data):
        image_id = data['image_id']
        try:
            image = api.image_get(request, image_id)
            flavor = api.flavor_get(request, data['flavor'])
            api.server_create(request,
                              data['name'],
                              image,
                              flavor,
                              user_data=data['user_data'],
                              key_name=data.get('key_name'))

            msg = 'Instance was successfully launched'
            LOG.info(msg)
            messages.success(request, msg)
            return shortcuts.redirect(request.build_absolute_uri())

        except api_exceptions.ApiException, e:
            LOG.error('ApiException while creating instances of image "%s"' %
                      image_id, exc_info=True)
            messages.error(request,
                           'Unable to launch instance: %s' % e.message)


@login_required
def index(request, tenant_id):
    tenant = api.token_get_tenant(request, request.user.tenant)
    all_images = []
    try:
        all_images = api.image_list_detailed(request)
        if not all_images:
            messages.info(request, "There are currently no images.")
    except glance_exception.ClientConnectionError, e:
        LOG.error("Error connecting to glance", exc_info=True)
        messages.error(request, "Error connecting to glance: %s" % str(e))
    except glance_exception.Error, e:
        LOG.error("Error retrieving image list", exc_info=True)
        messages.error(request, "Error retrieving image list: %s" % str(e))

    images = [im for im in all_images
              if im['container_format'] not in ['aki', 'ari']]

    return render_to_response('dash_images.html', {
        'tenant': tenant,
        'images': images,
    }, context_instance=template.RequestContext(request))


@login_required
def launch(request, tenant_id, image_id):
    def flavorlist():
        try:
            fl = api.flavor_list(request)

            # TODO add vcpu count to flavors
            sel = [(f.id, '%s (%svcpu / %sGB Disk / %sMB Ram )' %
                   (f.name, f.vcpus, f.disk, f.ram)) for f in fl]
            return sorted(sel)
        except api_exceptions.ApiException:
            LOG.error('Unable to retrieve list of instance types',
                      exc_info=True)
            return [(1, 'm1.tiny')]

    def keynamelist():
        try:
            fl = api.keypair_list(request)
            sel = [(f.key_name, f.key_name) for f in fl]
            return sel
        except api_exceptions.ApiException:
            LOG.error('Unable to retrieve list of keypairs', exc_info=True)
            return []

    # TODO(mgius): Any reason why these can't be after the launchform logic?
    # If The form is valid, we've just wasted these two api calls
    image = api.image_get(request, image_id)
    tenant = api.token_get_tenant(request, request.user.tenant)

    form, handled = LaunchForm.maybe_handle(
            request, initial={'flavorlist': flavorlist(),
                              'keynamelist': keynamelist(),
                              'image_id': image_id})
    if handled:
        return handled

    return render_to_response('dash_launch.html', {
        'tenant': tenant,
        'image': image,
        'form': form,
    }, context_instance=template.RequestContext(request))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Base classes for view based unit tests.
"""

import mox

from django import http
from django import shortcuts
from django import template as django_template
from django import test
from django.conf import settings
from django_openstack.middleware import keystone


class Object(object):
    """Inner Object for api resource wrappers"""
    pass


def fake_render_to_response(template_name, context, context_instance=None,
                            mimetype='text/html'):
    """Replacement for render_to_response so that views can be tested
       without having to stub out templates that belong in the frontend
       implementation.

       Should be able to be tested using the django unit test assertions like a
       normal render_to_response return value can be.
    """
    class Template(object):
        def __init__(self, name):
            self.name = name

    if context_instance is None:
        context_instance = django_template.Context(context)
    else:
        context_instance.update(context)

    resp = http.HttpResponse()
    template = Template(template_name)

    resp.write('<html><body><p>'
               'This is a fake httpresponse for testing purposes only'
               '</p></body></html>')

    # Allows django.test.client to populate fields on the response object
    test.signals.template_rendered.send(template, template=template,
                                        context=context_instance)

    return resp


class BaseViewTests(test.TestCase):
    TEST_PROJECT = 'test'
    TEST_REGION = 'test'
    TEST_STAFF_USER = 'staffUser'
    TEST_TENANT = 'aTenant'
    TEST_TOKEN = 'aToken'
    TEST_USER = 'test'

    @classmethod
    def setUpClass(cls):
        cls._real_render_to_response = shortcuts.render_to_response
        shortcuts.render_to_response = fake_render_to_response
        cls._real_get_user_from_request = keystone.get_user_from_request

    @classmethod
    def tearDownClass(cls):
        shortcuts.render_to_response = cls._real_render_to_response
        keystone.get_user_from_request = cls._real_get_user_from_request

    def setUp(self):
        self.mox = mox.Mox()
        self.setActiveUser(self.TEST_TOKEN, self.TEST_USER, self.TEST_TENANT,
                           True)

    def tearDown(self):
        self.mox.UnsetStubs()

    def assertRedirectsNoFollow(self, response, expected_url):
        self.assertEqual(response._headers['location'],
                         ('Location', settings.TESTSERVER + expected_url))
        self.assertEqual(response.status_code, 302)

    def setActiveUser(self, token, username, tenant, is_admin):
        keystone.get_user_from_request = \
                lambda x: keystone.User(token, username, tenant, is_admin)



from django import http
from django.contrib import messages
from django.core.urlresolvers import reverse
from django_openstack import api
from django_openstack.tests.view_tests import base
from mox import IsA

import openstackx.api.exceptions as api_exceptions

class KeyPairViewTests(base.BaseViewTests):
    def setUp(self):
        super(KeyPairViewTests, self).setUp()
        keypair_inner = base.Object()
        keypair_inner.key_name = 'keyName'
        self.keypairs = (api.KeyPair(keypair_inner),)

    def test_index(self):
        self.mox.StubOutWithMock(api, 'keypair_list')
        api.keypair_list(IsA(http.HttpRequest)).AndReturn(self.keypairs)

        self.mox.ReplayAll()

        res = self.client.get(reverse('dash_keypairs', args=[self.TEST_TENANT]))

        self.assertTemplateUsed(res, 'dash_keypairs.html')
        self.assertItemsEqual(res.context['keypairs'], self.keypairs)

        self.mox.VerifyAll()

    def test_index_exception(self):
        exception = api_exceptions.ApiException('apiException',
                                                message='apiException')
        self.mox.StubOutWithMock(api, 'keypair_list')
        api.keypair_list(IsA(http.HttpRequest)).AndRaise(exception)

        self.mox.StubOutWithMock(messages, 'error')
        messages.error(IsA(http.HttpRequest), IsA(str))

        self.mox.ReplayAll()

        res = self.client.get(reverse('dash_keypairs', args=[self.TEST_TENANT]))

        self.assertTemplateUsed(res, 'dash_keypairs.html')
        self.assertEqual(len(res.context['keypairs']), 0)

        self.mox.VerifyAll()

    def test_delete_keypair(self):
        KEYPAIR_ID = self.keypairs[0].key_name
        formData = {'method': 'DeleteKeypair',
                    'keypair_id': KEYPAIR_ID,
                    }

        self.mox.StubOutWithMock(api, 'keypair_delete')
        api.keypair_delete(IsA(http.HttpRequest), unicode(KEYPAIR_ID))

        self.mox.ReplayAll()

        res = self.client.post(reverse('dash_keypairs',
                                       args=[self.TEST_TENANT]),
                               formData)

        self.assertRedirectsNoFollow(res, reverse('dash_keypairs',
                                                  args=[self.TEST_TENANT]))

        self.mox.VerifyAll()

    def test_delete_keypair_exception(self):
        KEYPAIR_ID = self.keypairs[0].key_name
        formData = {'method': 'DeleteKeypair',
                    'keypair_id': KEYPAIR_ID,
                    }

        exception = api_exceptions.ApiException('apiException',
                                                message='apiException')
        self.mox.StubOutWithMock(api, 'keypair_delete')
        api.keypair_delete(IsA(http.HttpRequest),
                           unicode(KEYPAIR_ID)).AndRaise(exception)

        self.mox.ReplayAll()

        res = self.client.post(reverse('dash_keypairs',
                                       args=[self.TEST_TENANT]),
                               formData)

        self.assertRedirectsNoFollow(res, reverse('dash_keypairs',
                                                  args=[self.TEST_TENANT]))

        self.mox.VerifyAll()

    def test_create_keypair_get(self):
        res = self.client.get(reverse('dash_keypairs_create',
                                      args=[self.TEST_TENANT]))
        
        self.assertTemplateUsed(res, 'dash_keypairs_create.html')

    def test_create_keypair_post(self):
        KEYPAIR_NAME = 'newKeypair'
        PRIVATE_KEY = 'privateKey'

        newKeyPair = self.mox.CreateMock(api.KeyPair)
        newKeyPair.key_name = KEYPAIR_NAME
        newKeyPair.private_key = PRIVATE_KEY

        formData = {'method': 'CreateKeypair',
                    'name': KEYPAIR_NAME,
                    }

        self.mox.StubOutWithMock(api, 'keypair_create')
        api.keypair_create(IsA(http.HttpRequest),
                           KEYPAIR_NAME).AndReturn(newKeyPair)

        self.mox.ReplayAll()

        res = self.client.post(reverse('dash_keypairs_create',
                                       args=[self.TEST_TENANT]),
                               formData)

        self.assertTrue(res.has_header('Content-Disposition'))

        self.mox.VerifyAll()

    def test_create_keypair_exception(self):
        KEYPAIR_NAME = 'newKeypair'

        formData = {'method': 'CreateKeypair',
                    'name': KEYPAIR_NAME,
                    }

        exception = api_exceptions.ApiException('apiException',
                                                message='apiException')
        self.mox.StubOutWithMock(api, 'keypair_create')
        api.keypair_create(IsA(http.HttpRequest),
                           KEYPAIR_NAME).AndRaise(exception)

        self.mox.ReplayAll()

        res = self.client.post(reverse('dash_keypairs_create',
                                       args=[self.TEST_TENANT]),
                               formData)

        self.assertRedirectsNoFollow(res, reverse('dash_keypairs_create',
                                                  args=[self.TEST_TENANT]))

        self.mox.VerifyAll()



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django import template
from django import http
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.shortcuts import render_to_response
from django.shortcuts import redirect
from django.utils.translation import ugettext as _

import datetime
import json
import logging

from django.contrib import messages

from django_openstack import api
from django_openstack import forms
from django_openstack.dash.views import instances as dash_instances
from openstackx.api import exceptions as api_exceptions


LOG = logging.getLogger('django_openstack.syspanel.views.users')


class UserForm(forms.Form):
    def __init__(self, *args, **kwargs):
        tenant_list = kwargs.pop('tenant_list', None)
        super(UserForm, self).__init__(*args, **kwargs)
        self.fields['tenant_id'].choices = [[tenant.id,tenant.id] for tenant in tenant_list]

    id = forms.CharField(label="ID")
    email = forms.CharField(label="Email")
    password = forms.CharField(label="Password", widget=forms.PasswordInput(render_value=False), required=False)
    tenant_id = forms.ChoiceField(label="Primary Tenant")


class UserDeleteForm(forms.SelfHandlingForm):
    user = forms.CharField(required=True)

    def handle(self, request, data):
        user_id = data['user']
        LOG.info('Deleting user with id "%s"' % user_id)
        api.user_delete(request, user_id)
        messages.success(request,
                         '%s was successfully deleted.'
                         % user_id)
        return redirect(request.build_absolute_uri())


@login_required
def index(request):
    for f in (UserDeleteForm,):
        _, handled = f.maybe_handle(request)
        if handled:
            return handled

    users = api.user_list(request)

    user_delete_form = UserDeleteForm()
    return render_to_response('syspanel_users.html',{
        'users': users,
        'user_delete_form': user_delete_form,
    }, context_instance = template.RequestContext(request))


@login_required
def update(request, user_id):
    if request.method == "POST":
        tenants = api.tenant_list(request)
        form = UserForm(request.POST, tenant_list=tenants)
        if form.is_valid():
            user = form.clean()
            updated = []
            if user['email']:
                updated.append('email')
                api.user_update_email(request, user['id'], user['email'])
            if user['password']:
                updated.append('password')
                api.user_update_password(request, user['id'], user['password'])
            if user['tenant_id']:
                updated.append('tenant')
                api.user_update_tenant(request, user['id'], user['tenant_id'])
            messages.success(request,
                             'Updated %s for %s.'
                             % (', '.join(updated), user_id))
            return redirect('syspanel_users')
        else:
            # TODO add better error management
            messages.error(request, 'Unable to update user,\
                                    please try again.')

            return render_to_response(
            'syspanel_user_update.html',{
                'form': form,
                'user_id': user_id,
            }, context_instance = template.RequestContext(request))

    else:
        u = api.user_get(request, user_id)
        tenants = api.tenant_list(request)
        try:
            # FIXME
            email = u.email
        except:
            email = ''

        try:
            tenant_id = u.tenantId
        except:
            tenant_id = None
        form = UserForm(initial={'id': user_id,
                                 'tenant_id': tenant_id,
                                 'email': email},
                                 tenant_list=tenants)
        return render_to_response(
        'syspanel_user_update.html',{
            'form': form,
            'user_id': user_id,
        }, context_instance = template.RequestContext(request))


@login_required
def create(request):
    tenants = api.tenant_list(request)

    if request.method == "POST":
        form = UserForm(request.POST, tenant_list=tenants)
        if form.is_valid():
            user = form.clean()
            # TODO Make this a real request
            try:
                LOG.info('Creating user with id "%s"' % user['id'])
                api.user_create(request,
                                user['id'],
                                user['email'],
                                user['password'],
                                user['tenant_id'],
                                True)

                messages.success(request,
                                 '%s was successfully created.'
                                 % user['id'])
                return redirect('syspanel_users')

            except api_exceptions.ApiException, e:
                LOG.error('ApiException while creating user\n'
                          'id: "%s", email: "%s", tenant_id: "%s"' %
                          (user['id'], user['email'], user['tenant_id']),
                          exc_info=True)
                messages.error(request,
                                 'Error creating user: %s'
                                 % e.message)
                return redirect('syspanel_users')
        else:
            return render_to_response(
            'syspanel_user_create.html',{
                'form': form,
            }, context_instance = template.RequestContext(request))

    else:
        form = UserForm(tenant_list=tenants)
        return render_to_response(
        'syspanel_user_create.html',{
            'form': form,
        }, context_instance = template.RequestContext(request))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.contrib import messages
from django import shortcuts
import openstackx
import openstack


class User(object):
    def __init__(self, token, user, tenant, admin):
        self.token = token
        self.username = user
        self.tenant = tenant
        self.admin = admin

    def is_authenticated(self):
        # TODO: deal with token expiration
        return self.token

    def is_admin(self):
        return self.admin


def get_user_from_request(request):
    if 'user' not in request.session:
        return User(None,None,None,None)
    return User(request.session['token'],
                request.session['user'],
                request.session['tenant'],
                request.session['admin'])


class LazyUser(object):
    def __get__(self, request, obj_type=None):
        if not hasattr(request, '_cached_user'):
            request._cached_user = get_user_from_request(request)
        return request._cached_user


class AuthenticationMiddleware(object):
    def process_request(self, request):
        request.__class__.user = LazyUser()

    def process_exception(self, request, exception):
        if type(exception) in [openstack.compute.exceptions.Forbidden,
                               openstackx.api.exceptions.Forbidden]:
            messages.error(request, 'Your token has expired.\
                                     Please log in again')
            return shortcuts.redirect('/auth/logout')



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from operator import itemgetter

from django import template
from django import http
from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.shortcuts import redirect
from django.shortcuts import render_to_response
from openstackx.api import exceptions as api_exceptions


from django_openstack import api
from django_openstack import forms

LOG = logging.getLogger('django_openstack.syspanel.views.flavors')


class CreateFlavor(forms.SelfHandlingForm):
    flavorid = forms.CharField(max_length="10", label="Flavor ID")
    name = forms.CharField(max_length="5", label="Name")
    vcpus = forms.CharField(max_length="5", label="VCPUs")
    memory_mb = forms.CharField(max_length="5", label="Memory MB")
    disk_gb = forms.CharField(max_length="5", label="Disk GB")

    def handle(self, request, data):
        api.flavor_create(request,
                          data['name'],
                          int(data['memory_mb']),
                          int(data['vcpus']),
                          int(data['disk_gb']),
                          int(data['flavorid']))
        msg = '%s was successfully added to flavors.' % data['name']
        LOG.info(msg)
        messages.success(request, msg)
        return redirect('syspanel_flavors')


class DeleteFlavor(forms.SelfHandlingForm):
    flavorid = forms.CharField(required=True)

    def handle(self, request, data):
        flavor_id = data['flavorid']
        LOG.info('Deleting flavor with id "%s"' % flavor_id)
        api.flavor_delete(request, flavor_id, True)
        return redirect(request.build_absolute_uri())


@login_required
def index(request):
    for f in (DeleteFlavor,):
        _, handled = f.maybe_handle(request)
        if handled:
            return handled

    # We don't have any way of showing errors for these, so don't bother
    # trying to reuse the forms from above
    delete_form = DeleteFlavor()

    flavors = []
    try:
        flavors = api.flavor_list(request)
    except api_exceptions.ApiException, e:
        LOG.error('ApiException while fetching usage info', exc_info=True)
        messages.error(request, 'Unable to get usage info: %s' % e.message)

    flavors.sort(key=lambda x: x.id, reverse=True)
    return render_to_response('syspanel_flavors.html',{
        'delete_form': delete_form,
        'flavors': flavors,
    }, context_instance = template.RequestContext(request))


@login_required
def create(request):
    form, handled = CreateFlavor.maybe_handle(request)
    if handled:
        return handled

    return render_to_response('syspanel_create_flavor.html',{
        'form': form,
    }, context_instance = template.RequestContext(request))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Tests for dependency packages
Honestly, this can probably go away once tests that depend on these
packages become more ingrained in the code.
"""

from django import test
from django.core import mail
from mailer import engine
from mailer import send_mail


class DjangoMailerPresenceTest(test.TestCase):
    def test_mailsent(self):
        send_mail('subject', 'message_body', 'from@test.com', ['to@test.com'])
        engine.send_all()

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, 'subject')



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Views for managing Nova instances.
"""
import logging

from django import http
from django import template
from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django import shortcuts
from django.utils.translation import ugettext as _

from django_openstack import api
from django_openstack import forms
import openstackx.api.exceptions as api_exceptions


LOG = logging.getLogger('django_openstack.dash.views.keypairs')


class DeleteKeypair(forms.SelfHandlingForm):
    keypair_id = forms.CharField(widget=forms.HiddenInput())

    def handle(self, request, data):
        try:
            LOG.info('Deleting keypair "%s"' % data['keypair_id'])
            keypair = api.keypair_delete(request, data['keypair_id'])
            messages.info(request, 'Successfully deleted keypair: %s' \
                                    % data['keypair_id'])
        except api_exceptions.ApiException, e:
            LOG.error("ApiException in DeleteKeypair", exc_info=True)
            messages.error(request, 'Error deleting keypair: %s' % e.message)
        return shortcuts.redirect(request.build_absolute_uri())


class CreateKeypair(forms.SelfHandlingForm):
    name = forms.CharField(max_length="20", label="Keypair Name")

    def handle(self, request, data):
        try:
            LOG.info('Creating keypair "%s"' % data['name'])
            keypair = api.keypair_create(request, data['name'])
            response = http.HttpResponse(mimetype='application/binary')
            response['Content-Disposition'] = \
                'attachment; filename=%s.pem' % \
                keypair.key_name
            response.write(keypair.private_key)
            return response
        except api_exceptions.ApiException, e:
            LOG.error("ApiException in CreateKeyPair", exc_info=True)
            messages.error(request, 'Error Creating Keypair: %s' % e.message)
            return shortcuts.redirect(request.build_absolute_uri())


@login_required
def index(request, tenant_id):
    delete_form, handled = DeleteKeypair.maybe_handle(request)
    if handled:
        return handled

    try:
        keypairs = api.keypair_list(request)
    except api_exceptions.ApiException, e:
        keypairs = []
        LOG.error("ApiException in keypair index", exc_info=True)
        messages.error(request, 'Error fetching keypairs: %s' % e.message)

    return shortcuts.render_to_response('dash_keypairs.html', {
        'keypairs': keypairs,
        'delete_form': delete_form,
    }, context_instance=template.RequestContext(request))


@login_required
def create(request, tenant_id):
    form, handled = CreateKeypair.maybe_handle(request)
    if handled:
        return handled

    return shortcuts.render_to_response('dash_keypairs_create.html', {
        'create_form': form,
    }, context_instance=template.RequestContext(request))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import cloudfiles
import httplib
import json
import mox

from django import http
from django import test
from django.conf import settings
from django_openstack import api
from glance import client as glance_client
from mox import IsA
from openstack import compute as OSCompute
from openstackx import admin as OSAdmin
from openstackx import auth as OSAuth
from openstackx import extras as OSExtras


TEST_CONSOLE_KIND = 'vnc'
TEST_EMAIL = 'test@test.com'
TEST_HOSTNAME = 'hostname'
TEST_INSTANCE_ID = '2'
TEST_PASSWORD = '12345'
TEST_PORT = 8000
TEST_RETURN = 'retValue'
TEST_TENANT_DESCRIPTION = 'tenantDescription'
TEST_TENANT_ID = '1234'
TEST_TOKEN = 'aToken'
TEST_TOKEN_ID = 'userId'
TEST_URL = 'http://%s:%s/something/v1.0' % (TEST_HOSTNAME, TEST_PORT)
TEST_USERNAME = 'testUser'


class Server(object):
    """ More or less fakes what the api is looking for """
    def __init__(self, id, imageRef, attrs=None):
        self.id = id
        self.imageRef = imageRef
        if attrs is not None:
            self.attrs = attrs

    def __eq__(self, other):
        if self.id != other.id or \
            self.imageRef != other.imageRef:
                return False

        for k in self.attrs:
            if other.attrs.__getattr__(k) != v:
                return False

        return True

    def __ne__(self, other):
        return not self == other


class Tenant(object):
    """ More or less fakes what the api is looking for """
    def __init__(self, id, description, enabled):
        self.id = id
        self.description = description
        self.enabled = enabled

    def __eq__(self, other):
        return self.id == other.id and \
               self.description == other.description and \
               self.enabled == other.enabled

    def __ne__(self, other):
        return not self == other


class Token(object):
    """ More or less fakes what the api is looking for """
    def __init__(self, id, username, tenant_id, serviceCatalog=None):
        self.id = id
        self.username = username
        self.tenant_id = tenant_id
        self.serviceCatalog = serviceCatalog

    def __eq__(self, other):
        return self.id == other.id and \
               self.username == other.username and \
               self.tenant_id == other.tenant_id and \
               self.serviceCatalog == other.serviceCatalog

    def __ne__(self, other):
        return not self == other


class APIResource(api.APIResourceWrapper):
    """ Simple APIResource for testing """
    _attrs = ['foo', 'bar', 'baz']

    @staticmethod
    def get_instance(innerObject=None):
        if innerObject is None:
            class InnerAPIResource(object):
                pass
            innerObject = InnerAPIResource()
            innerObject.foo = 'foo'
            innerObject.bar = 'bar'
        return APIResource(innerObject)


class APIDict(api.APIDictWrapper):
    """ Simple APIDict for testing """
    _attrs = ['foo', 'bar', 'baz']

    @staticmethod
    def get_instance(innerDict=None):
        if innerDict is None:
            innerDict = {'foo': 'foo',
                         'bar': 'bar'}
        return APIDict(innerDict)


class APIResourceWrapperTests(test.TestCase):
    def test_get_attribute(self):
        resource = APIResource.get_instance()
        self.assertEqual(resource.foo, 'foo')

    def test_get_invalid_attribute(self):
        resource = APIResource.get_instance()
        self.assertNotIn('missing', resource._attrs,
                msg="Test assumption broken.  Find new missing attribute")
        with self.assertRaises(AttributeError):
            resource.missing

    def test_get_inner_missing_attribute(self):
        resource = APIResource.get_instance()
        with self.assertRaises(AttributeError):
            resource.baz


class APIDictWrapperTests(test.TestCase):
    # APIDict allows for both attribute access and dictionary style [element]
    # style access.  Test both
    def test_get_item(self):
        resource = APIDict.get_instance()
        self.assertEqual(resource.foo, 'foo')
        self.assertEqual(resource['foo'], 'foo')

    def test_get_invalid_item(self):
        resource = APIDict.get_instance()
        self.assertNotIn('missing', resource._attrs,
                msg="Test assumption broken.  Find new missing attribute")
        with self.assertRaises(AttributeError):
            resource.missing
        with self.assertRaises(KeyError):
            resource['missing']

    def test_get_inner_missing_attribute(self):
        resource = APIDict.get_instance()
        with self.assertRaises(AttributeError):
            resource.baz
        with self.assertRaises(KeyError):
            resource['baz']

    def test_get_with_default(self):
        resource = APIDict.get_instance()

        self.assertEqual(resource.get('foo'), 'foo')

        self.assertIsNone(resource.get('baz'))

        self.assertEqual('retValue', resource.get('baz', 'retValue'))


# Wrapper classes that only define _attrs don't need extra testing.
# Wrapper classes that have other attributes or methods need testing
class ImageWrapperTests(test.TestCase):
    dict_with_properties = {
            'properties':
                {'image_state': 'running'},
            'size': 100,
            }
    dict_without_properties = {
            'size': 100,
            }

    def test_get_properties(self):
        image = api.Image(self.dict_with_properties)
        image_props = image.properties
        self.assertIsInstance(image_props, api.ImageProperties)
        self.assertEqual(image_props.image_state, 'running')

    def test_get_other(self):
        image = api.Image(self.dict_with_properties)
        self.assertEqual(image.size, 100)

    def test_get_properties_missing(self):
        image = api.Image(self.dict_without_properties)
        with self.assertRaises(AttributeError):
            image.properties

    def test_get_other_missing(self):
        image = api.Image(self.dict_without_properties)
        with self.assertRaises(AttributeError):
            self.assertNotIn('missing', image._attrs,
                msg="Test assumption broken.  Find new missing attribute")
            image.missing


class ServerWrapperTests(test.TestCase):
    HOST = 'hostname'
    ID = '1'
    IMAGE_NAME = 'imageName'
    IMAGE_REF = '3'

    def setUp(self):
        self.mox = mox.Mox()

        # these are all objects "fetched" from the api
        self.inner_attrs = {'host': self.HOST}

        self.inner_server = Server(self.ID, self.IMAGE_REF, self.inner_attrs)
        self.inner_server_no_attrs = Server(self.ID, self.IMAGE_REF)

        self.request = self.mox.CreateMock(http.HttpRequest)

    def tearDown(self):
        self.mox.UnsetStubs()

    def test_get_attrs(self):
        server = api.Server(self.inner_server, self.request)
        attrs = server.attrs
        # for every attribute in the "inner" object passed to the api wrapper,
        # see if it can be accessed through the api.ServerAttribute instance
        for k in self.inner_attrs:
            self.assertEqual(attrs.__getattr__(k), self.inner_attrs[k])

    def test_get_other(self):
        server = api.Server(self.inner_server, self.request)
        self.assertEqual(server.id, self.ID)

    def test_get_attrs_missing(self):
        server = api.Server(self.inner_server_no_attrs, self.request)
        with self.assertRaises(AttributeError):
            server.attrs

    def test_get_other_missing(self):
        server = api.Server(self.inner_server, self.request)
        with self.assertRaises(AttributeError):
            self.assertNotIn('missing', server._attrs,
                msg="Test assumption broken.  Find new missing attribute")
            server.missing

    def test_image_name(self):
        self.mox.StubOutWithMock(api, 'image_get')
        api.image_get(IsA(http.HttpRequest),
                      self.IMAGE_REF
                      ).AndReturn(api.Image({'name': self.IMAGE_NAME}))

        server = api.Server(self.inner_server, self.request)

        self.mox.ReplayAll()

        image_name = server.image_name

        self.assertEqual(image_name, self.IMAGE_NAME)

        self.mox.VerifyAll()


class ApiHelperTests(test.TestCase):
    """ Tests for functions that don't use one of the api objects """
    def setUp(self):
        self.mox = mox.Mox()
        self.request = http.HttpRequest()
        self.request.session = dict()

    def tearDown(self):
        self.mox.UnsetStubs()

    def test_url_for(self):
        GLANCE_URL = 'http://glance/glanceapi/'
        NOVA_URL = 'http://nova/novapi/'

        serviceCatalog = {
                'glance': [{'adminURL': GLANCE_URL + 'admin',
                            'internalURL': GLANCE_URL + 'internal'},
                          ],
                'nova': [{'adminURL': NOVA_URL + 'admin',
                          'internalURL': NOVA_URL + 'internal'},
                        ],
                }

        self.request.session['serviceCatalog'] = serviceCatalog

        url = api.url_for(self.request, 'glance')
        self.assertEqual(url, GLANCE_URL + 'internal')

        url = api.url_for(self.request, 'glance', admin=False)
        self.assertEqual(url, GLANCE_URL + 'internal')

        url = api.url_for(self.request, 'glance', admin=True)
        self.assertEqual(url, GLANCE_URL + 'admin')

        url = api.url_for(self.request, 'nova')
        self.assertEqual(url, NOVA_URL + 'internal')

        url = api.url_for(self.request, 'nova', admin=False)
        self.assertEqual(url, NOVA_URL + 'internal')

        url = api.url_for(self.request, 'nova', admin=True)
        self.assertEqual(url, NOVA_URL + 'admin')

    def test_token_info(self):
        """ This function uses the keystone api, but not through an
            api client, because there doesn't appear to be one for
            keystone
        """
        GLANCE_URL = 'http://glance/glance_api/'
        KEYSTONE_HOST = 'keystonehost'
        KEYSTONE_PORT = 8080
        KEYSTONE_URL = 'http://%s:%d/keystone/' % (KEYSTONE_HOST,
                                                   KEYSTONE_PORT)

        serviceCatalog = {
                'glance': [{'adminURL': GLANCE_URL + 'admin',
                            'internalURL': GLANCE_URL + 'internal'},
                          ],
                'identity': [{'adminURL': KEYSTONE_URL + 'admin',
                          'internalURL': KEYSTONE_URL + 'internal'},
                        ],
                }

        token = Token(TEST_TOKEN_ID, TEST_TENANT_ID,
                      TEST_USERNAME, serviceCatalog)

        jsonData = {
                'auth': {
                    'token': {
                        'expires': '2011-07-02T02:01:19.382655',
                        'id': '3c5748d5-bec6-4215-843a-f959d589f4b0',
                        },
                    'user': {
                        'username': 'joeuser',
                        'roleRefs': [{'roleId': 'Minion'}],
                        'tenantId': u'1234'
                        }
                    }
                }

        jsonDataAdmin = {
                'auth': {
                    'token': {
                        'expires': '2011-07-02T02:01:19.382655',
                        'id': '3c5748d5-bec6-4215-843a-f959d589f4b0',
                        },
                    'user': {
                        'username': 'joeuser',
                        'roleRefs': [{'roleId': 'Admin'}],
                        'tenantId': u'1234'
                        }
                    }
                }

        # setup test where user is not admin
        self.mox.StubOutClassWithMocks(httplib, 'HTTPConnection')

        conn = httplib.HTTPConnection(KEYSTONE_HOST, KEYSTONE_PORT)
        response = self.mox.CreateMock(httplib.HTTPResponse)

        conn.request(IsA(str), IsA(str), headers=IsA(dict))
        conn.getresponse().AndReturn(response)

        response.read().AndReturn(json.dumps(jsonData))

        expected_nonadmin_val = {
                'tenant': '1234',
                'user': 'joeuser',
                'admin': False
                }

        # setup test where user is admin
        conn = httplib.HTTPConnection(KEYSTONE_HOST, KEYSTONE_PORT)
        response = self.mox.CreateMock(httplib.HTTPResponse)

        conn.request(IsA(str), IsA(str), headers=IsA(dict))
        conn.getresponse().AndReturn(response)

        response.read().AndReturn(json.dumps(jsonDataAdmin))

        expected_admin_val = {
                'tenant': '1234',
                'user': 'joeuser',
                'admin': True
                }

        self.mox.ReplayAll()

        ret_val = api.token_info(None, token)

        self.assertDictEqual(ret_val, expected_nonadmin_val)

        ret_val = api.token_info(None, token)

        self.assertDictEqual(ret_val, expected_admin_val)

        self.mox.VerifyAll()


class AccountApiTests(test.TestCase):
    def setUp(self):
        self.mox = mox.Mox()
        self.request = http.HttpRequest()
        self.request.session = dict()
        self.request.session['token'] = TEST_TOKEN

    def tearDown(self):
        self.mox.UnsetStubs()

    def stub_account_api(self):
        self.mox.StubOutWithMock(api, 'account_api')
        account_api = self.mox.CreateMock(OSExtras.Account)
        api.account_api(IsA(http.HttpRequest)).AndReturn(account_api)
        return account_api

    def test_get_account_api(self):
        self.mox.StubOutClassWithMocks(OSExtras, 'Account')
        OSExtras.Account(auth_token=TEST_TOKEN, management_url=TEST_URL)

        self.mox.StubOutWithMock(api, 'url_for')
        api.url_for(
                IsA(http.HttpRequest), 'keystone', True).AndReturn(TEST_URL)
        api.url_for(
                IsA(http.HttpRequest), 'keystone', True).AndReturn(TEST_URL)

        self.mox.ReplayAll()

        self.assertIsNotNone(api.account_api(self.request))

        self.mox.VerifyAll()

    def test_tenant_create(self):
        DESCRIPTION = 'aDescription'
        ENABLED = True

        account_api = self.stub_account_api()

        account_api.tenants = self.mox.CreateMockAnything()
        account_api.tenants.create(TEST_TENANT_ID, DESCRIPTION,
                                   ENABLED).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.tenant_create(self.request, TEST_TENANT_ID,
                                    DESCRIPTION, ENABLED)

        self.assertIsInstance(ret_val, api.Tenant)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()

    def test_tenant_get(self):
        account_api = self.stub_account_api()

        account_api.tenants = self.mox.CreateMockAnything()
        account_api.tenants.get(TEST_TENANT_ID).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.tenant_get(self.request, TEST_TENANT_ID)

        self.assertIsInstance(ret_val, api.Tenant)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()

    def test_tenant_list(self):
        tenants = (TEST_RETURN, TEST_RETURN + '2')

        account_api = self.stub_account_api()

        account_api.tenants = self.mox.CreateMockAnything()
        account_api.tenants.list().AndReturn(tenants)

        self.mox.ReplayAll()

        ret_val = api.tenant_list(self.request)

        self.assertEqual(len(ret_val), len(tenants))
        for tenant in ret_val:
            self.assertIsInstance(tenant, api.Tenant)
            self.assertIn(tenant._apiresource, tenants)

        self.mox.VerifyAll()

    def test_tenant_update(self):
        DESCRIPTION = 'aDescription'
        ENABLED = True

        account_api = self.stub_account_api()

        account_api.tenants = self.mox.CreateMockAnything()
        account_api.tenants.update(TEST_TENANT_ID, DESCRIPTION,
                                   ENABLED).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.tenant_update(self.request, TEST_TENANT_ID,
                                    DESCRIPTION, ENABLED)

        self.assertIsInstance(ret_val, api.Tenant)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()

    def test_user_create(self):
        account_api = self.stub_account_api()

        account_api.users = self.mox.CreateMockAnything()
        account_api.users.create(TEST_USERNAME, TEST_EMAIL, TEST_PASSWORD,
                                TEST_TENANT_ID).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.user_create(self.request, TEST_USERNAME, TEST_EMAIL,
                                  TEST_PASSWORD, TEST_TENANT_ID)

        self.assertIsInstance(ret_val, api.User)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()

    def test_user_delete(self):
        account_api = self.stub_account_api()

        account_api.users = self.mox.CreateMockAnything()
        account_api.users.delete(TEST_USERNAME).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.user_delete(self.request, TEST_USERNAME)

        self.assertIsNone(ret_val)

        self.mox.VerifyAll()

    def test_user_get(self):
        account_api = self.stub_account_api()

        account_api.users = self.mox.CreateMockAnything()
        account_api.users.get(TEST_USERNAME).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.user_get(self.request, TEST_USERNAME)

        self.assertIsInstance(ret_val, api.User)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()

    def test_user_list(self):
        users = (TEST_USERNAME, TEST_USERNAME + '2')

        account_api = self.stub_account_api()
        account_api.users = self.mox.CreateMockAnything()
        account_api.users.list().AndReturn(users)

        self.mox.ReplayAll()

        ret_val = api.user_list(self.request)

        self.assertEqual(len(ret_val), len(users))
        for user in ret_val:
            self.assertIsInstance(user, api.User)
            self.assertIn(user._apiresource, users)

        self.mox.VerifyAll()

    def test_user_update_email(self):
        account_api = self.stub_account_api()
        account_api.users = self.mox.CreateMockAnything()
        account_api.users.update_email(TEST_USERNAME,
                                       TEST_EMAIL).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.user_update_email(self.request, TEST_USERNAME,
                                        TEST_EMAIL)

        self.assertIsInstance(ret_val, api.User)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()

    def test_user_update_password(self):
        account_api = self.stub_account_api()
        account_api.users = self.mox.CreateMockAnything()
        account_api.users.update_password(TEST_USERNAME,
                                          TEST_PASSWORD).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.user_update_password(self.request, TEST_USERNAME,
                                           TEST_PASSWORD)

        self.assertIsInstance(ret_val, api.User)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()

    def test_user_update_tenant(self):
        account_api = self.stub_account_api()
        account_api.users = self.mox.CreateMockAnything()
        account_api.users.update_tenant(TEST_USERNAME,
                                        TEST_TENANT_ID).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.user_update_tenant(self.request, TEST_USERNAME,
                                           TEST_TENANT_ID)

        self.assertIsInstance(ret_val, api.User)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()


class AdminApiTests(test.TestCase):
    def setUp(self):
        self.mox = mox.Mox()
        self.request = http.HttpRequest()
        self.request.session = dict()
        self.request.session['token'] = TEST_TOKEN

    def tearDown(self):
        self.mox.UnsetStubs()

    def stub_admin_api(self, count=1):
        self.mox.StubOutWithMock(api, 'admin_api')
        admin_api = self.mox.CreateMock(OSAdmin.Admin)
        for i in range(count):
            api.admin_api(IsA(http.HttpRequest)).AndReturn(admin_api)
        return admin_api

    def test_get_admin_api(self):
        self.mox.StubOutClassWithMocks(OSAdmin, 'Admin')
        OSAdmin.Admin(auth_token=TEST_TOKEN, management_url=TEST_URL)

        self.mox.StubOutWithMock(api, 'url_for')
        api.url_for(IsA(http.HttpRequest), 'nova', True).AndReturn(TEST_URL)
        api.url_for(IsA(http.HttpRequest), 'nova', True).AndReturn(TEST_URL)

        self.mox.ReplayAll()

        self.assertIsNotNone(api.admin_api(self.request))

        self.mox.VerifyAll()

    def test_flavor_create(self):
        FLAVOR_DISK = 1000
        FLAVOR_ID = 6
        FLAVOR_MEMORY = 1024
        FLAVOR_NAME = 'newFlavor'
        FLAVOR_VCPU = 2

        admin_api = self.stub_admin_api()

        admin_api.flavors = self.mox.CreateMockAnything()
        admin_api.flavors.create(FLAVOR_NAME, FLAVOR_MEMORY, FLAVOR_VCPU,
                                 FLAVOR_DISK, FLAVOR_ID).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.flavor_create(self.request, FLAVOR_NAME,
                                    str(FLAVOR_MEMORY), str(FLAVOR_VCPU),
                                    str(FLAVOR_DISK), FLAVOR_ID)

        self.assertIsInstance(ret_val, api.Flavor)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()

    def test_flavor_delete(self):
        FLAVOR_ID = 6

        admin_api = self.stub_admin_api(count=2)

        admin_api.flavors = self.mox.CreateMockAnything()
        admin_api.flavors.delete(FLAVOR_ID, False).AndReturn(TEST_RETURN)
        admin_api.flavors.delete(FLAVOR_ID, True).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.flavor_delete(self.request, FLAVOR_ID)
        self.assertIsNone(ret_val)

        ret_val = api.flavor_delete(self.request, FLAVOR_ID, purge=True)
        self.assertIsNone(ret_val)

    def test_service_get(self):
        NAME = 'serviceName'

        admin_api = self.stub_admin_api()
        admin_api.services = self.mox.CreateMockAnything()
        admin_api.services.get(NAME).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.service_get(self.request, NAME)

        self.assertIsInstance(ret_val, api.Services)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()

    def test_service_list(self):
        services = (TEST_RETURN, TEST_RETURN + '2')

        admin_api = self.stub_admin_api()
        admin_api.services = self.mox.CreateMockAnything()
        admin_api.services.list().AndReturn(services)

        self.mox.ReplayAll()

        ret_val = api.service_list(self.request)

        for service in ret_val:
            self.assertIsInstance(service, api.Services)
            self.assertIn(service._apiresource, services)

        self.mox.VerifyAll()

    def test_service_update(self):
        ENABLED = True
        NAME = 'serviceName'

        admin_api = self.stub_admin_api()
        admin_api.services = self.mox.CreateMockAnything()
        admin_api.services.update(NAME, ENABLED).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.service_update(self.request, NAME, ENABLED)

        self.assertIsInstance(ret_val, api.Services)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()


class AuthApiTests(test.TestCase):
    def setUp(self):
        self.mox = mox.Mox()

    def tearDown(self):
        self.mox.UnsetStubs()

    def test_get_auth_api(self):
        settings.OPENSTACK_KEYSTONE_URL = TEST_URL
        self.mox.StubOutClassWithMocks(OSAuth, 'Auth')
        OSAuth.Auth(management_url=settings.OPENSTACK_KEYSTONE_URL)

        self.mox.ReplayAll()

        self.assertIsNotNone(api.auth_api())

        self.mox.VerifyAll()

    def test_token_get_tenant(self):
        self.mox.StubOutWithMock(api, 'auth_api')
        auth_api_mock = self.mox.CreateMockAnything()
        api.auth_api().AndReturn(auth_api_mock)

        tenants_mock = self.mox.CreateMockAnything()
        auth_api_mock.tenants = tenants_mock

        tenant_list = [Tenant('notTheDroid',
                              'notTheDroid_desc',
                              False),
                       Tenant(TEST_TENANT_ID,
                              TEST_TENANT_DESCRIPTION,
                              True),
                      ]
        tenants_mock.for_token('aToken').AndReturn(tenant_list)

        request_mock = self.mox.CreateMock(http.HttpRequest)
        request_mock.session = {'token': 'aToken'}

        self.mox.ReplayAll()

        ret_val = api.token_get_tenant(request_mock, TEST_TENANT_ID)
        self.assertEqual(tenant_list[1], ret_val)

        self.mox.VerifyAll()

    def test_token_get_tenant_no_tenant(self):
        self.mox.StubOutWithMock(api, 'auth_api')
        auth_api_mock = self.mox.CreateMockAnything()
        api.auth_api().AndReturn(auth_api_mock)

        tenants_mock = self.mox.CreateMockAnything()
        auth_api_mock.tenants = tenants_mock

        tenant_list = [Tenant('notTheDroid',
                              'notTheDroid_desc',
                              False),
                      ]
        tenants_mock.for_token('aToken').AndReturn(tenant_list)

        request_mock = self.mox.CreateMock(http.HttpRequest)
        request_mock.session = {'token': 'aToken'}

        self.mox.ReplayAll()

        ret_val = api.token_get_tenant(request_mock, TEST_TENANT_ID)
        self.assertIsNone(ret_val)

        self.mox.VerifyAll()

    def test_token_list_tenants(self):
        self.mox.StubOutWithMock(api, 'auth_api')
        auth_api_mock = self.mox.CreateMockAnything()
        api.auth_api().AndReturn(auth_api_mock)

        tenants_mock = self.mox.CreateMockAnything()
        auth_api_mock.tenants = tenants_mock

        tenant_list = [Tenant('notTheDroid',
                              'notTheDroid_desc',
                              False),
                       Tenant(TEST_TENANT_ID,
                              TEST_TENANT_DESCRIPTION,
                              True),
                      ]
        tenants_mock.for_token('aToken').AndReturn(tenant_list)

        request_mock = self.mox.CreateMock(http.HttpRequest)

        self.mox.ReplayAll()

        ret_val = api.token_list_tenants(request_mock, 'aToken')
        for tenant in ret_val:
            self.assertIn(tenant, tenant_list)

        self.mox.VerifyAll()

    def test_token_create(self):
        self.mox.StubOutWithMock(api, 'auth_api')
        auth_api_mock = self.mox.CreateMockAnything()
        api.auth_api().AndReturn(auth_api_mock)

        tokens_mock = self.mox.CreateMockAnything()
        auth_api_mock.tokens = tokens_mock

        test_token = Token(TEST_TOKEN_ID, TEST_USERNAME, TEST_TENANT_ID)

        tokens_mock.create(TEST_TENANT_ID, TEST_USERNAME,
                           TEST_PASSWORD).AndReturn(test_token)

        request_mock = self.mox.CreateMock(http.HttpRequest)

        self.mox.ReplayAll()

        ret_val = api.token_create(request_mock, TEST_TENANT_ID,
                                   TEST_USERNAME, TEST_PASSWORD)

        self.assertEqual(test_token, ret_val)

        self.mox.VerifyAll()


class ComputeApiTests(test.TestCase):
    def setUp(self):
        self.mox = mox.Mox()
        self.request = http.HttpRequest()
        self.request.session = {}
        self.request.session['token'] = TEST_TOKEN

    def tearDown(self):
        self.mox.UnsetStubs()

    def stub_compute_api(self, count=1):
        self.mox.StubOutWithMock(api, 'compute_api')
        compute_api = self.mox.CreateMock(OSCompute.Compute)
        for i in range(count):
            api.compute_api(IsA(http.HttpRequest)).AndReturn(compute_api)
        return compute_api

    def test_get_compute_api(self):
        class ComputeClient(object):
            __slots__ = ['auth_token', 'management_url']

        self.mox.StubOutClassWithMocks(OSCompute, 'Compute')
        compute_api = OSCompute.Compute(auth_token=TEST_TOKEN,
                                        management_url=TEST_URL)

        compute_api.client = ComputeClient()

        self.mox.StubOutWithMock(api, 'url_for')
        # called three times?  Looks like a good place for optimization
        api.url_for(IsA(http.HttpRequest), 'nova').AndReturn(TEST_URL)
        api.url_for(IsA(http.HttpRequest), 'nova').AndReturn(TEST_URL)
        api.url_for(IsA(http.HttpRequest), 'nova').AndReturn(TEST_URL)

        self.mox.ReplayAll()

        compute_api = api.compute_api(self.request)

        self.assertIsNotNone(compute_api)
        self.assertEqual(compute_api.client.auth_token, TEST_TOKEN)
        self.assertEqual(compute_api.client.management_url, TEST_URL)

        self.mox.VerifyAll()

    def test_flavor_get(self):
        FLAVOR_ID = 6

        compute_api = self.stub_compute_api()

        compute_api.flavors = self.mox.CreateMockAnything()
        compute_api.flavors.get(FLAVOR_ID).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.flavor_get(self.request, FLAVOR_ID)
        self.assertIsInstance(ret_val, api.Flavor)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()

    def test_server_delete(self):
        INSTANCE = 'anInstance'

        compute_api = self.stub_compute_api()

        compute_api.servers = self.mox.CreateMockAnything()
        compute_api.servers.delete(INSTANCE).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.server_delete(self.request, INSTANCE)

        self.assertIsNone(ret_val)

        self.mox.VerifyAll()

    def test_server_get(self):
        INSTANCE_ID = '2'

        compute_api = self.stub_compute_api()
        compute_api.servers = self.mox.CreateMockAnything()
        compute_api.servers.get(INSTANCE_ID).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.server_get(self.request, INSTANCE_ID)

        self.assertIsInstance(ret_val, api.Server)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()

    def test_server_reboot(self):
        INSTANCE_ID = '2'
        HARDNESS = 'diamond'

        self.mox.StubOutWithMock(api, 'server_get')

        server = self.mox.CreateMock(OSCompute.Server)
        server.reboot(OSCompute.servers.REBOOT_HARD).AndReturn(TEST_RETURN)
        api.server_get(IsA(http.HttpRequest), INSTANCE_ID).AndReturn(server)

        server = self.mox.CreateMock(OSCompute.Server)
        server.reboot(HARDNESS).AndReturn(TEST_RETURN)
        api.server_get(IsA(http.HttpRequest), INSTANCE_ID).AndReturn(server)

        self.mox.ReplayAll()

        ret_val = api.server_reboot(self.request, INSTANCE_ID)
        self.assertIsNone(ret_val)

        ret_val = api.server_reboot(self.request, INSTANCE_ID,
                                    hardness=HARDNESS)
        self.assertIsNone(ret_val)

        self.mox.VerifyAll()


class ExtrasApiTests(test.TestCase):
    def setUp(self):
        self.mox = mox.Mox()
        self.request = http.HttpRequest()
        self.request.session = dict()
        self.request.session['token'] = TEST_TOKEN

    def tearDown(self):
        self.mox.UnsetStubs()

    def stub_extras_api(self, count=1):
        self.mox.StubOutWithMock(api, 'extras_api')
        extras_api = self.mox.CreateMock(OSExtras.Extras)
        for i in range(count):
            api.extras_api(IsA(http.HttpRequest)).AndReturn(extras_api)
        return extras_api

    def test_get_extras_api(self):
        self.mox.StubOutClassWithMocks(OSExtras, 'Extras')
        OSExtras.Extras(auth_token=TEST_TOKEN, management_url=TEST_URL)

        self.mox.StubOutWithMock(api, 'url_for')
        api.url_for(IsA(http.HttpRequest), 'nova').AndReturn(TEST_URL)
        api.url_for(IsA(http.HttpRequest), 'nova').AndReturn(TEST_URL)

        self.mox.ReplayAll()

        self.assertIsNotNone(api.extras_api(self.request))

        self.mox.VerifyAll()

    def test_console_create(self):
        extras_api = self.stub_extras_api(count=2)
        extras_api.consoles = self.mox.CreateMockAnything()
        extras_api.consoles.create(
                TEST_INSTANCE_ID, TEST_CONSOLE_KIND).AndReturn(TEST_RETURN)
        extras_api.consoles.create(
                TEST_INSTANCE_ID, None).AndReturn(TEST_RETURN + '2')

        self.mox.ReplayAll()

        ret_val = api.console_create(self.request,
                                     TEST_INSTANCE_ID,
                                     TEST_CONSOLE_KIND)
        self.assertIsInstance(ret_val, api.Console)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        ret_val = api.console_create(self.request, TEST_INSTANCE_ID)
        self.assertIsInstance(ret_val, api.Console)
        self.assertEqual(ret_val._apiresource, TEST_RETURN + '2')

        self.mox.VerifyAll()

    def test_flavor_list(self):
        flavors = (TEST_RETURN, TEST_RETURN + '2')
        extras_api = self.stub_extras_api()
        extras_api.flavors = self.mox.CreateMockAnything()
        extras_api.flavors.list().AndReturn(flavors)

        self.mox.ReplayAll()

        ret_val = api.flavor_list(self.request)

        self.assertEqual(len(ret_val), len(flavors))
        for flavor in ret_val:
            self.assertIsInstance(flavor, api.Flavor)
            self.assertIn(flavor._apiresource, flavors)

        self.mox.VerifyAll()

    def test_keypair_create(self):
        NAME = '1'

        extras_api = self.stub_extras_api()
        extras_api.keypairs = self.mox.CreateMockAnything()
        extras_api.keypairs.create(NAME).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.keypair_create(self.request, NAME)
        self.assertIsInstance(ret_val, api.KeyPair)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()

    def test_keypair_delete(self):
        KEYPAIR_ID = '1'

        extras_api = self.stub_extras_api()
        extras_api.keypairs = self.mox.CreateMockAnything()
        extras_api.keypairs.delete(KEYPAIR_ID).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.keypair_delete(self.request, KEYPAIR_ID)
        self.assertIsNone(ret_val)

        self.mox.VerifyAll()

    def test_keypair_list(self):
        NAME = 'keypair'
        keypairs = (NAME + '1', NAME + '2')

        extras_api = self.stub_extras_api()
        extras_api.keypairs = self.mox.CreateMockAnything()
        extras_api.keypairs.list().AndReturn(keypairs)

        self.mox.ReplayAll()

        ret_val = api.keypair_list(self.request)

        self.assertEqual(len(ret_val), len(keypairs))
        for keypair in ret_val:
            self.assertIsInstance(keypair, api.KeyPair)
            self.assertIn(keypair._apiresource, keypairs)

        self.mox.VerifyAll()

    def test_server_create(self):
        NAME = 'server'
        IMAGE = 'anImage'
        FLAVOR = 'cherry'
        USER_DATA = {'nuts': 'berries'}
        KEY = 'user'

        extras_api = self.stub_extras_api()
        extras_api.servers = self.mox.CreateMockAnything()
        extras_api.servers.create(NAME, IMAGE, FLAVOR, user_data=USER_DATA,
                                  key_name=KEY).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.server_create(self.request, NAME, IMAGE, FLAVOR,
                                    USER_DATA, KEY)

        self.assertIsInstance(ret_val, api.Server)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()

    def test_server_list(self):
        servers = (TEST_RETURN, TEST_RETURN + '2')

        extras_api = self.stub_extras_api()

        extras_api.servers = self.mox.CreateMockAnything()
        extras_api.servers.list().AndReturn(servers)

        self.mox.ReplayAll()

        ret_val = api.server_list(self.request)

        self.assertEqual(len(ret_val), len(servers))
        for server in ret_val:
            self.assertIsInstance(server, api.Server)
            self.assertIn(server._apiresource, servers)

        self.mox.VerifyAll()

    def test_usage_get(self):
        extras_api = self.stub_extras_api()

        extras_api.usage = self.mox.CreateMockAnything()
        extras_api.usage.get(TEST_TENANT_ID, 'start',
                             'end').AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.usage_get(self.request, TEST_TENANT_ID, 'start', 'end')

        self.assertIsInstance(ret_val, api.Usage)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()

    def test_usage_list(self):
        usages = (TEST_RETURN, TEST_RETURN + '2')

        extras_api = self.stub_extras_api()

        extras_api.usage = self.mox.CreateMockAnything()
        extras_api.usage.list('start', 'end').AndReturn(usages)

        self.mox.ReplayAll()

        ret_val = api.usage_list(self.request, 'start', 'end')

        self.assertEqual(len(ret_val), len(usages))
        for usage in ret_val:
            self.assertIsInstance(usage, api.Usage)
            self.assertIn(usage._apiresource, usages)

        self.mox.VerifyAll()


class GlanceApiTests(test.TestCase):
    def setUp(self):
        self.mox = mox.Mox()

        self.request = http.HttpRequest()
        self.request.session = dict()
        self.request.session['token'] = TEST_TOKEN

    def tearDown(self):
        self.mox.UnsetStubs()

    def stub_glance_api(self, count=1):
        self.mox.StubOutWithMock(api, 'glance_api')
        glance_api = self.mox.CreateMock(glance_client.Client)
        for i in range(count):
            api.glance_api(IsA(http.HttpRequest)).AndReturn(glance_api)
        return glance_api

    def test_get_glance_api(self):
        self.mox.StubOutClassWithMocks(glance_client, 'Client')
        glance_client.Client(TEST_HOSTNAME, TEST_PORT)

        self.mox.StubOutWithMock(api, 'url_for')
        api.url_for(IsA(http.HttpRequest), 'glance').AndReturn(TEST_URL)

        self.mox.ReplayAll()

        self.assertIsNotNone(api.glance_api(self.request))

        self.mox.VerifyAll()

    def test_image_create(self):
        IMAGE_FILE = 'someData'
        IMAGE_META = {'metadata': 'foo'}

        glance_api = self.stub_glance_api()
        glance_api.add_image(IMAGE_META, IMAGE_FILE).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.image_create(self.request, IMAGE_META, IMAGE_FILE)

        self.assertIsInstance(ret_val, api.Image)
        self.assertEqual(ret_val._apidict, TEST_RETURN)

        self.mox.VerifyAll()

    def test_image_delete(self):
        IMAGE_ID = '1'

        glance_api = self.stub_glance_api()
        glance_api.delete_image(IMAGE_ID).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.image_delete(self.request, IMAGE_ID)

        self.assertEqual(ret_val, TEST_RETURN)

        self.mox.VerifyAll()

    def test_image_get(self):
        IMAGE_ID = '1'

        glance_api = self.stub_glance_api()
        glance_api.get_image(IMAGE_ID).AndReturn([TEST_RETURN])

        self.mox.ReplayAll()

        ret_val = api.image_get(self.request, IMAGE_ID)

        self.assertIsInstance(ret_val, api.Image)
        self.assertEqual(ret_val._apidict, TEST_RETURN)

    def test_image_list_detailed(self):
        images = (TEST_RETURN, TEST_RETURN + '2')
        glance_api = self.stub_glance_api()
        glance_api.get_images_detailed().AndReturn(images)

        self.mox.ReplayAll()

        ret_val = api.image_list_detailed(self.request)

        self.assertEqual(len(ret_val), len(images))
        for image in ret_val:
            self.assertIsInstance(image, api.Image)
            self.assertIn(image._apidict, images)

        self.mox.VerifyAll()

    def test_image_update(self):
        IMAGE_ID = '1'
        IMAGE_META = {'metadata': 'foobar'}

        glance_api = self.stub_glance_api(count=2)
        glance_api.update_image(IMAGE_ID, image_meta={}).AndReturn(TEST_RETURN)
        glance_api.update_image(IMAGE_ID,
                                image_meta=IMAGE_META).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.image_update(self.request, IMAGE_ID)

        self.assertIsInstance(ret_val, api.Image)
        self.assertEqual(ret_val._apidict, TEST_RETURN)

        ret_val = api.image_update(self.request,
                                   IMAGE_ID,
                                   image_meta=IMAGE_META)

        self.assertIsInstance(ret_val, api.Image)
        self.assertEqual(ret_val._apidict, TEST_RETURN)

        self.mox.VerifyAll()


class SwiftApiTests(test.TestCase):
    def setUp(self):
        self.mox = mox.Mox()

    def tearDown(self):
        self.mox.UnsetStubs()

    def stub_swift_api(self, count=1):
        self.mox.StubOutWithMock(api, 'swift_api')
        swift_api = self.mox.CreateMock(cloudfiles.connection.Connection)
        for i in range(count):
            api.swift_api().AndReturn(swift_api)
        return swift_api

    def test_get_swift_api(self):
        self.mox.StubOutWithMock(cloudfiles, 'get_connection')

        swiftuser = ':'.join((settings.SWIFT_ACCOUNT, settings.SWIFT_USER))
        cloudfiles.get_connection(swiftuser,
                                  settings.SWIFT_PASS,
                                  authurl=settings.SWIFT_AUTHURL
                                 ).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        self.assertEqual(api.swift_api(), TEST_RETURN)

        self.mox.VerifyAll()

    def test_swift_get_containers(self):
        containers = (TEST_RETURN, TEST_RETURN + '2')

        swift_api = self.stub_swift_api()

        swift_api.get_all_containers().AndReturn(containers)

        self.mox.ReplayAll()

        ret_val = api.swift_get_containers()

        self.assertEqual(len(ret_val), len(containers))
        for container in ret_val:
            self.assertIsInstance(container, api.Container)
            self.assertIn(container._apiresource, containers)

        self.mox.VerifyAll()

    def test_swift_create_container(self):
        NAME = 'containerName'

        swift_api = self.stub_swift_api()
        self.mox.StubOutWithMock(api, 'swift_container_exists')

        api.swift_container_exists(NAME).AndReturn(False)
        swift_api.create_container(NAME).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.swift_create_container(NAME)

        self.assertIsInstance(ret_val, api.Container)
        self.assertEqual(ret_val._apiresource, TEST_RETURN)

        self.mox.VerifyAll()

    def test_swift_delete_container(self):
        NAME = 'containerName'

        swift_api = self.stub_swift_api()

        swift_api.delete_container(NAME).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.swift_delete_container(NAME)

        self.assertIsNone(ret_val)

        self.mox.VerifyAll()

    def test_swift_get_objects(self):
        NAME = 'containerName'

        swift_objects = (TEST_RETURN, TEST_RETURN + '2')
        container = self.mox.CreateMock(cloudfiles.container.Container)
        container.get_objects().AndReturn(swift_objects)

        swift_api = self.stub_swift_api()

        swift_api.get_container(NAME).AndReturn(container)

        self.mox.ReplayAll()

        ret_val = api.swift_get_objects(NAME)

        self.assertEqual(len(ret_val), len(swift_objects))
        for swift_object in ret_val:
            self.assertIsInstance(swift_object, api.SwiftObject)
            self.assertIn(swift_object._apiresource, swift_objects)

        self.mox.VerifyAll()

    def test_swift_upload_object(self):
        CONTAINER_NAME = 'containerName'
        OBJECT_NAME = 'objectName'
        OBJECT_DATA = 'someData'

        swift_api = self.stub_swift_api()
        container = self.mox.CreateMock(cloudfiles.container.Container)
        swift_object = self.mox.CreateMock(cloudfiles.storage_object.Object)

        swift_api.get_container(CONTAINER_NAME).AndReturn(container)
        container.create_object(OBJECT_NAME).AndReturn(swift_object)
        swift_object.write(OBJECT_DATA).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.swift_upload_object(CONTAINER_NAME, OBJECT_NAME,
                                          OBJECT_DATA)

        self.assertIsNone(ret_val)

        self.mox.VerifyAll()

    def test_swift_delete_object(self):
        CONTAINER_NAME = 'containerName'
        OBJECT_NAME = 'objectName'

        swift_api = self.stub_swift_api()
        container = self.mox.CreateMock(cloudfiles.container.Container)

        swift_api.get_container(CONTAINER_NAME).AndReturn(container)
        container.delete_object(OBJECT_NAME).AndReturn(TEST_RETURN)

        self.mox.ReplayAll()

        ret_val = api.swift_delete_object(CONTAINER_NAME, OBJECT_NAME)

        self.assertIsNone(ret_val)

        self.mox.VerifyAll()

    def test_swift_get_object_data(self):
        CONTAINER_NAME = 'containerName'
        OBJECT_NAME = 'objectName'
        OBJECT_DATA = 'objectData'

        swift_api = self.stub_swift_api()
        container = self.mox.CreateMock(cloudfiles.container.Container)
        swift_object = self.mox.CreateMock(cloudfiles.storage_object.Object)

        swift_api.get_container(CONTAINER_NAME).AndReturn(container)
        container.get_object(OBJECT_NAME).AndReturn(swift_object)
        swift_object.stream().AndReturn(OBJECT_DATA)

        self.mox.ReplayAll()

        ret_val = api.swift_get_object_data(CONTAINER_NAME, OBJECT_NAME)

        self.assertEqual(ret_val, OBJECT_DATA)

        self.mox.VerifyAll()

    def test_swift_object_exists(self):
        CONTAINER_NAME = 'containerName'
        OBJECT_NAME = 'objectName'

        swift_api = self.stub_swift_api()
        container = self.mox.CreateMock(cloudfiles.container.Container)
        swift_object = self.mox.CreateMock(cloudfiles.Object)

        swift_api.get_container(CONTAINER_NAME).AndReturn(container)
        container.get_object(OBJECT_NAME).AndReturn(swift_object)

        self.mox.ReplayAll()

        ret_val = api.swift_object_exists(CONTAINER_NAME, OBJECT_NAME)
        self.assertTrue(ret_val)

        self.mox.VerifyAll()

    def test_swift_copy_object(self):
        CONTAINER_NAME = 'containerName'
        OBJECT_NAME = 'objectName'

        swift_api = self.stub_swift_api()
        container = self.mox.CreateMock(cloudfiles.container.Container)
        self.mox.StubOutWithMock(api, 'swift_object_exists')

        swift_object = self.mox.CreateMock(cloudfiles.Object)

        swift_api.get_container(CONTAINER_NAME).AndReturn(container)
        api.swift_object_exists(CONTAINER_NAME, OBJECT_NAME).AndReturn(False)

        container.get_object(OBJECT_NAME).AndReturn(swift_object)
        swift_object.copy_to(CONTAINER_NAME, OBJECT_NAME)

        self.mox.ReplayAll()

        ret_val = api.swift_copy_object(CONTAINER_NAME, OBJECT_NAME,
                                        CONTAINER_NAME, OBJECT_NAME)

        self.assertIsNone(ret_val)
        self.mox.VerifyAll()



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Unit tests for instance views.
"""

import boto.ec2.instance
import mox

from django.core.urlresolvers import reverse
from django_openstack.nova.tests.base import (BaseProjectViewTests,
                                              TEST_PROJECT)


TEST_INSTANCE_ID = 'i-abcdefgh'


class InstanceViewTests(BaseProjectViewTests):
    def test_index(self):
        self.mox.StubOutWithMock(self.project, 'get_instances')
        self.project.get_instances().AndReturn([])

        self.mox.ReplayAll()

        res = self.client.get(reverse('nova_instances', args=[TEST_PROJECT]))
        self.assertEqual(res.status_code, 200)
        self.assertTemplateUsed(res,
                'django_openstack/nova/instances/index.html')
        self.assertEqual(len(res.context['instances']), 0)

        self.mox.VerifyAll()

    def test_detail(self):
        instance = boto.ec2.instance.Instance()
        instance.id = TEST_INSTANCE_ID
        instance.displayName = instance.id
        instance.displayDescription = instance.id

        self.mox.StubOutWithMock(self.project, 'get_instance')
        self.project.get_instance(instance.id).AndReturn(instance)
        self.mox.StubOutWithMock(self.project, 'get_instances')
        self.project.get_instances().AndReturn([instance])

        self.mox.ReplayAll()

        res = self.client.get(reverse('nova_instances_detail',
                                     args=[TEST_PROJECT, TEST_INSTANCE_ID]))
        self.assertEqual(res.status_code, 200)
        self.assertTemplateUsed(res,
                'django_openstack/nova/instances/index.html')
        self.assertEqual(res.context['selected_instance'].id, instance.id)

        self.mox.VerifyAll()



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import logging
import re

from django import utils
from django.conf import settings
from django.contrib import messages
from django.forms import widgets
from django.utils import dates
from django.utils import safestring
from django.utils import formats

from django.forms import *

LOG = logging.getLogger('django_openstack.forms')
RE_DATE = re.compile(r'(\d{4})-(\d\d?)-(\d\d?)$')


class SelectDateWidget(widgets.Widget):
    """
    A Widget that splits date input into three <select> boxes.

    This also serves as an example of a Widget that has more than one HTML
    element and hence implements value_from_datadict.
    """
    none_value = (0, '---')
    month_field = '%s_month'
    day_field = '%s_day'
    year_field = '%s_year'

    def __init__(self, attrs=None, years=None, required=True, skip_day_field=False):
        # years is an optional list/tuple of years to use in the "year" select box.
        self.attrs = attrs or {}
        self.required = required
        self.skip_day_field = skip_day_field
        if years:
            self.years = years
        else:
            this_year = datetime.date.today().year
            self.years = range(this_year, this_year+10)

    def render(self, name, value, attrs=None, skip_day_field=True):
        print "Render %s %s" % (name, value)
        try:
            year_val, month_val, day_val = value.year, value.month, value.day
        except AttributeError:
            year_val = month_val = day_val = None
            if isinstance(value, basestring):
                if settings.USE_L10N:
                    try:
                        input_format = formats.get_format('DATE_INPUT_FORMATS')[0]
                        # Python 2.4 compatibility:
                        #     v = datetime.datetime.strptime(value, input_format)
                        # would be clearer, but datetime.strptime was added in
                        # Python 2.5
                        v = datetime.datetime(*(time.strptime(value, input_format)[0:6]))
                        year_val, month_val, day_val = v.year, v.month, v.day
                    except ValueError:
                        pass
                else:
                    match = RE_DATE.match(value)
                    if match:
                        year_val, month_val, day_val = [int(v) for v in match.groups()]
        choices = [(i, i) for i in self.years]
        year_html = self.create_select(name, self.year_field, value, year_val, choices)
        choices = dates.MONTHS.items()
        month_html = self.create_select(name, self.month_field, value, month_val, choices)
        choices = [(i, i) for i in range(1, 32)]
        day_html = self.create_select(name, self.day_field, value, day_val,  choices)

        format = formats.get_format('DATE_FORMAT')
        escaped = False
        output = []
        for char in format:
            if escaped:
                escaped = False
            elif char == '\\':
                escaped = True
            elif char in 'Yy':
                output.append(year_html)
            elif char in 'bFMmNn':
                output.append(month_html)
            elif char in 'dj' and not self.skip_day_field:
                output.append(day_html)
        return safestring.mark_safe(u'\n'.join(output))

    def id_for_label(self, id_):
        return '%s_month' % id_
    id_for_label = classmethod(id_for_label)

    def value_from_datadict(self, data, files, name):
        y = data.get(self.year_field % name)
        m = data.get(self.month_field % name)
        d = data.get(self.day_field % name)
        if y == m == d == "0":
            return None
        if y and m and d:
            if settings.USE_L10N:
                input_format = formats.get_format('DATE_INPUT_FORMATS')[0]
                try:
                    date_value = datetime.date(int(y), int(m), int(d))
                except ValueError:
                    pass
                else:
                    date_value = utils.datetime_safe.new_date(date_value)
                    return date_value.strftime(input_format)
            else:
                return '%s-%s-%s' % (y, m, d)
        return data.get(name, None)

    def create_select(self, name, field, value, val, choices):
        if 'id' in self.attrs:
            id_ = self.attrs['id']
        else:
            id_ = 'id_%s' % name
        if not (self.required and val):
            choices.insert(0, self.none_value)
        local_attrs = self.build_attrs(id=field % id_)
        s = widgets.Select(choices=choices)
        select_html = s.render(field % name, val, local_attrs)
        return select_html


class SelfHandlingForm(Form):
    method = CharField(required=True, widget=HiddenInput)

    def __init__(self, *args, **kwargs):
        initial = kwargs.pop('initial', {})
        initial['method'] = self.__class__.__name__
        kwargs['initial'] = initial
        super(SelfHandlingForm, self).__init__(*args, **kwargs)

    @classmethod
    def maybe_handle(cls, request, *args, **kwargs):
        if cls.__name__ != request.POST.get('method'):
            return cls(*args, **kwargs), None

        try:
            if request.FILES:
                form = cls(request.POST, request.FILES, *args, **kwargs)
            else:
                form = cls(request.POST, *args, **kwargs)

            if not form.is_valid():
                return form, None

            data = form.clean()

            return form, form.handle(request, data)
        except Exception as e:
            LOG.error('Nonspecific error while handling form', exc_info=True)
            messages.error(request, 'Unexpected error: %s' % e.message)
            return form, None


class DateForm(Form):
    date = DateField(widget=SelectDateWidget(years=range(datetime.date.today().year, 2009, -1), skip_day_field=True))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.conf.urls.defaults import *
from django.conf import settings


urlpatterns = patterns('django_openstack.auth.views',
    url(r'login/$', 'login', name='auth_login'),
    url(r'logout/$', 'logout', name='auth_logout'),
    url(r'switch/(?P<tenant_id>[^/]+)/$', 'switch_tenants', name='auth_switch'),
)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django import template
from django import http
from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.shortcuts import render_to_response
from django.shortcuts import redirect
from django.utils.translation import ugettext as _

import datetime
import json
import logging

from django.contrib import messages

from django_openstack import api
from django_openstack import forms
from django_openstack.dash.views import instances as dash_instances
from openstackx.api import exceptions as api_exceptions


LOG = logging.getLogger('django_openstack.syspanel.views.tenants')


class CreateTenant(forms.SelfHandlingForm):
    id = forms.CharField(label="ID (name)")
    description = forms.CharField(widget=forms.widgets.Textarea(), label="Description")
    enabled = forms.BooleanField(label="Enabled", required=False)

    def handle(self, request, data):
        try:
            LOG.info('Creating tenant with id "%s"' % data['id'])
            api.tenant_create(request,
                              data['id'],
                              data['description'],
                              data['enabled'])
            messages.success(request,
                             '%s was successfully created.'
                             % data['id'])
        except api_exceptions.ApiException, e:
            LOG.error('ApiException while creating tenant\n'
                      'Id: "%s", Description: "%s", Enabled "%s"' %
                      (data['id'], data['description'], data['enabled']),
                      exc_info=True)
            messages.error(request, 'Unable to create tenant: %s' %
                           (e.message))
        return redirect('syspanel_tenants')


class UpdateTenant(forms.SelfHandlingForm):
    id = forms.CharField(label="ID (name)", widget=forms.TextInput(attrs={'readonly':'readonly'}))
    description = forms.CharField(widget=forms.widgets.Textarea(), label="Description")
    enabled = forms.BooleanField(label="Enabled", required=False)

    def handle(self, request, data):
        try:
            LOG.info('Updating tenant with id "%s"' % data['id'])
            api.tenant_update(request,
                              data['id'],
                              data['description'],
                              data['enabled'])
            messages.success(request,
                             '%s was successfully updated.'
                             % data['id'])
        except api_exceptions.ApiException, e:
            LOG.error('ApiException while updating tenant\n'
                      'Id: "%s", Description: "%s", Enabled "%s"' %
                      (data['id'], data['description'], data['enabled']),
                      exc_info=True)
            messages.error(request, 'Unable to update tenant: %s' % e.message)
        return redirect('syspanel_tenants')


@login_required
def index(request):
    tenants = []
    try:
        tenants = api.tenant_list(request)
    except api_exceptions.ApiException, e:
        LOG.error('ApiException while getting tenant list', exc_info=True)
        messages.error(request, 'Unable to get tenant info: %s' % e.message)
    tenants.sort(key=lambda x: x.id, reverse=True)
    return render_to_response('syspanel_tenants.html',{
        'tenants': tenants,
    }, context_instance = template.RequestContext(request))


@login_required
def create(request):
    form, handled = CreateTenant.maybe_handle(request)
    if handled:
        return handled

    return render_to_response(
    'syspanel_tenant_create.html',{
        'form': form,
    }, context_instance = template.RequestContext(request))


@login_required
def update(request, tenant_id):
    form, handled = UpdateTenant.maybe_handle(request)
    if handled:
        return handled

    if request.method == 'GET':
        try:
            tenant = api.tenant_get(request, tenant_id)
            form = UpdateTenant(initial={'id': tenant.id,
                                         'description': tenant.description,
                                         'enabled': tenant.enabled})
        except api_exceptions.ApiException, e:
            LOG.error('Error fetching tenant with id "%s"' % tenant_id,
                      exc_info=True)
            messages.error(request, 'Unable to update tenant: %s' % e.message)
            return redirect('syspanel_tenants')

    return render_to_response(
    'syspanel_tenant_update.html',{
        'form': form,
    }, context_instance = template.RequestContext(request))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.conf.urls.defaults import *

INSTANCES = r'^(?P<tenant_id>[^/]+)/instances/(?P<instance_id>[^/]+)/%s$'
IMAGES = r'^(?P<tenant_id>[^/]+)/images/(?P<image_id>[^/]+)/%s$'
KEYPAIRS = r'^(?P<tenant_id>[^/]+)/keypairs/%s$'
CONTAINERS = r'^(?P<tenant_id>[^/]+)/containers/%s$'
OBJECTS = r'^(?P<tenant_id>[^/]+)/containers/(?P<container_name>[^/]+)/%s$'

urlpatterns = patterns('django_openstack.dash.views.instances',
    url(r'^(?P<tenant_id>[^/]+)/$', 'usage', name='dash_usage'),
    url(r'^(?P<tenant_id>[^/]+)/instances/$', 'index', name='dash_instances'),
    url(INSTANCES % 'console', 'console', name='dash_instances_console'),
    url(INSTANCES % 'vnc', 'vnc', name='dash_instances_vnc'),
)

urlpatterns += patterns('django_openstack.dash.views.images',
    url(r'^(?P<tenant_id>[^/]+)/images/$', 'index', name='dash_images'),
    url(IMAGES % 'launch', 'launch', name='dash_images_launch'),
)

urlpatterns += patterns('django_openstack.dash.views.keypairs',
    url(r'^(?P<tenant_id>[^/]+)/keypairs/$', 'index', name='dash_keypairs'),
    url(KEYPAIRS % 'create', 'create', name='dash_keypairs_create'),
)

# Swift containers and objects.
urlpatterns += patterns('django_openstack.dash.views.containers',
    url(CONTAINERS % '', 'index', name='dash_containers'),
    url(CONTAINERS % 'create', 'create', name='dash_containers_create'),
)

urlpatterns += patterns('django_openstack.dash.views.objects',
    url(OBJECTS % '', 'index', name='dash_objects'),
    url(OBJECTS % 'upload', 'upload', name='dash_objects_upload'),
    url(OBJECTS % '(?P<object_name>[^/]+)/copy',
        'copy', name='dash_object_copy'),
    url(OBJECTS % '(?P<object_name>[^/]+)/download',
        'download', name='dash_objects_download'),
)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Unit tests for image views.
"""

import boto.ec2.image
import boto.ec2.instance
import mox

from django.core.urlresolvers import reverse
from django_openstack.nova import forms
from django_openstack.nova import shortcuts
from django_openstack.nova.tests.base import (BaseProjectViewTests,
                                              TEST_PROJECT)


TEST_IMAGE_ID = 'ami_test'
TEST_INSTANCE_ID = 'i-abcdefg'
TEST_KEY = 'foo'


class ImageViewTests(BaseProjectViewTests):
    def setUp(self):
        self.ami = boto.ec2.image.Image()
        self.ami.id = TEST_IMAGE_ID
        setattr(self.ami, 'displayName', TEST_IMAGE_ID)
        setattr(self.ami, 'description', TEST_IMAGE_ID)
        super(ImageViewTests, self).setUp()

    def test_index(self):
        self.mox.StubOutWithMock(self.project, 'get_images')
        self.mox.StubOutWithMock(forms, 'get_key_pair_choices')
        self.mox.StubOutWithMock(forms, 'get_instance_type_choices')

        self.project.get_images().AndReturn([])
        forms.get_key_pair_choices(self.project).AndReturn([])
        forms.get_instance_type_choices().AndReturn([])

        self.mox.ReplayAll()

        res = self.client.get(reverse('nova_images', args=[TEST_PROJECT]))
        self.assertEqual(res.status_code, 200)
        self.assertTemplateUsed(res, 'django_openstack/nova/images/index.html')
        self.assertEqual(len(res.context['image_lists']), 3)

        self.mox.VerifyAll()

    def test_launch_form(self):
        self.mox.StubOutWithMock(self.project, 'get_image')
        self.mox.StubOutWithMock(forms, 'get_key_pair_choices')
        self.mox.StubOutWithMock(forms, 'get_instance_type_choices')

        self.project.get_image(TEST_IMAGE_ID).AndReturn(self.ami)
        forms.get_key_pair_choices(self.project).AndReturn([])
        forms.get_instance_type_choices().AndReturn([])

        self.mox.ReplayAll()

        args = [TEST_PROJECT, TEST_IMAGE_ID]
        res = self.client.get(reverse('nova_images_launch', args=args))
        self.assertEqual(res.status_code, 200)
        self.assertTemplateUsed(res,
                'django_openstack/nova/images/launch.html')
        self.assertEqual(res.context['ami'].id, TEST_IMAGE_ID)

        self.mox.VerifyAll()

    def test_launch(self):
        instance = boto.ec2.instance.Instance()
        instance.id = TEST_INSTANCE_ID
        instance.image_id = TEST_IMAGE_ID
        reservation = boto.ec2.instance.Reservation()
        reservation.instances = [instance]

        conn = self.mox.CreateMockAnything()

        self.mox.StubOutWithMock(forms, 'get_key_pair_choices')
        self.mox.StubOutWithMock(forms, 'get_instance_type_choices')
        self.mox.StubOutWithMock(self.project, 'get_openstack_connection')

        self.project.get_openstack_connection().AndReturn(conn)

        forms.get_key_pair_choices(self.project).AndReturn(
                self.create_key_pair_choices([TEST_KEY]))
        forms.get_instance_type_choices().AndReturn(
                self.create_instance_type_choices())

        params = {'addressing_type': 'private',
                'UserData': '', 'display_name': u'name',
                'MinCount': '1', 'key_name': TEST_KEY,
                'MaxCount': '1', 'InstanceType': 'm1.medium',
                'ImageId': TEST_IMAGE_ID}
        conn.get_object('RunInstances', params, boto.ec2.instance.Reservation,
                verb='POST').AndReturn(reservation)

        self.mox.ReplayAll()

        url = reverse('nova_images_launch', args=[TEST_PROJECT, TEST_IMAGE_ID])
        data = {'key_name': TEST_KEY,
                'count': '1',
                'size': 'm1.medium',
                'display_name': 'name',
                'user_data': ''}
        res = self.client.post(url, data)
        self.assertRedirectsNoFollow(res, reverse('nova_instances',
                                                  args=[TEST_PROJECT]))
        self.mox.VerifyAll()

    def test_detail(self):
        self.mox.StubOutWithMock(self.project, 'get_images')
        self.mox.StubOutWithMock(self.project, 'get_image')
        self.mox.StubOutWithMock(shortcuts, 'get_user_image_permissions')
        self.mox.StubOutWithMock(forms, 'get_key_pair_choices')
        self.mox.StubOutWithMock(forms, 'get_instance_type_choices')

        self.project.get_images().AndReturn([self.ami])
        self.project.get_image(TEST_IMAGE_ID).AndReturn(self.ami)
        forms.get_key_pair_choices(self.project).AndReturn(
            self.create_key_pair_choices([TEST_KEY]))
        forms.get_instance_type_choices().AndReturn(
            self.create_instance_type_choices())

        self.mox.ReplayAll()

        res = self.client.get(reverse('nova_images_detail',
                                      args=[TEST_PROJECT, TEST_IMAGE_ID]))
        self.assertEqual(res.status_code, 200)
        self.assertTemplateUsed(res, 'django_openstack/nova/images/index.html')
        self.assertEqual(res.context['ami'].id, TEST_IMAGE_ID)

        self.mox.VerifyAll()

    def test_remove_form(self):
        self.mox.StubOutWithMock(self.project, 'get_image')
        self.mox.ReplayAll()

        res = self.client.get(reverse('nova_images_remove',
                                      args=[TEST_PROJECT, TEST_IMAGE_ID]))
        self.assertRedirectsNoFollow(res, reverse('nova_images',
                                                  args=[TEST_PROJECT]))
        self.mox.VerifyAll()

    def test_remove(self):
        self.mox.StubOutWithMock(self.project, 'deregister_image')
        self.project.deregister_image(TEST_IMAGE_ID).AndReturn(True)
        self.mox.ReplayAll()

        res = self.client.post(reverse('nova_images_remove',
                                       args=[TEST_PROJECT, TEST_IMAGE_ID]))
        self.assertRedirectsNoFollow(res, reverse('nova_images',
                                                  args=[TEST_PROJECT]))

        self.mox.VerifyAll()

    def test_make_public(self):
        self.mox.StubOutWithMock(self.project, 'get_image')
        self.mox.StubOutWithMock(self.project, 'modify_image_attribute')

        self.ami.is_public = False
        self.project.get_image(TEST_IMAGE_ID).AndReturn(self.ami)
        self.project.modify_image_attribute(TEST_IMAGE_ID,
                                            attribute='launchPermission',
                                            operation='add').AndReturn(True)
        self.mox.ReplayAll()

        res = self.client.post(reverse('nova_images_privacy',
                                       args=[TEST_PROJECT, TEST_IMAGE_ID]))
        self.assertRedirectsNoFollow(res, reverse('nova_images_detail',
                                          args=[TEST_PROJECT, TEST_IMAGE_ID]))
        self.mox.VerifyAll()

    def test_make_private(self):
        self.mox.StubOutWithMock(self.project, 'get_image')
        self.mox.StubOutWithMock(self.project, 'modify_image_attribute')

        self.ami.is_public = True
        self.project.get_image(TEST_IMAGE_ID).AndReturn(self.ami)
        self.project.modify_image_attribute(TEST_IMAGE_ID,
                                            attribute='launchPermission',
                                            operation='remove').AndReturn(True)
        self.mox.ReplayAll()

        args = [TEST_PROJECT, TEST_IMAGE_ID]
        res = self.client.post(reverse('nova_images_privacy', args=args))
        self.assertRedirectsNoFollow(res, reverse('nova_images_detail',
                                                  args=args))
        self.mox.VerifyAll()

    def test_update_form(self):
        self.mox.StubOutWithMock(self.project, 'get_image')
        self.project.get_image(TEST_IMAGE_ID).AndReturn(self.ami)
        self.mox.ReplayAll()

        args = [TEST_PROJECT, TEST_IMAGE_ID]
        res = self.client.get(reverse('nova_images_update', args=args))
        self.assertEqual(res.status_code, 200)
        self.assertTemplateUsed(res, 'django_openstack/nova/images/edit.html')
        self.assertEqual(res.context['ami'].id, TEST_IMAGE_ID)

        self.mox.VerifyAll()

    def test_update(self):
        self.mox.StubOutWithMock(self.project, 'get_image')
        self.mox.StubOutWithMock(self.project, 'update_image')

        self.project.get_image(TEST_IMAGE_ID).AndReturn(self.ami)
        self.project.update_image(TEST_IMAGE_ID, 'test', 'test') \
                .AndReturn(True)

        self.mox.ReplayAll()

        args = [TEST_PROJECT, TEST_IMAGE_ID]
        data = {'nickname': 'test',
                'description': 'test'}
        url = reverse('nova_images_update', args=args)
        res = self.client.post(url, data)
        expected_url = reverse('nova_images_detail', args=args)
        self.assertRedirectsNoFollow(res, expected_url)

        self.mox.VerifyAll()



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2011 Fourth Paradigm Development, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Views for managing Swift containers.
"""
import logging

from django import template
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django import shortcuts

from django_openstack import api
from django_openstack import forms

from cloudfiles.errors import ContainerNotEmpty


LOG = logging.getLogger('django_openstack.dash')


class DeleteContainer(forms.SelfHandlingForm):
    container_name = forms.CharField(widget=forms.HiddenInput())

    def handle(self, request, data):
        try:
            api.swift_delete_container(data['container_name'])
        except ContainerNotEmpty, e:
            messages.error(request,
                           'Unable to delete non-empty container: %s' % \
                           data['container_name'])
            LOG.error('Unable to delete container "%s".  Exception: "%s"' %
                      (data['container_name'], str(e)))
        else:
            messages.info(request,
                      'Successfully deleted container: %s' % \
                      data['container_name'])
        return shortcuts.redirect(request.build_absolute_uri())


class CreateContainer(forms.SelfHandlingForm):
    name = forms.CharField(max_length="255", label="Container Name")

    def handle(self, request, data):
        api.swift_create_container(data['name'])
        messages.success(request, "Container was successfully created.")
        return shortcuts.redirect(request.build_absolute_uri())


@login_required
def index(request, tenant_id):
    delete_form, handled = DeleteContainer.maybe_handle(request)
    if handled:
        return handled

    containers = api.swift_get_containers()

    return shortcuts.render_to_response('dash_containers.html', {
        'containers': containers,
        'delete_form': delete_form,
    }, context_instance=template.RequestContext(request))


@login_required
def create(request, tenant_id):
    form, handled = CreateContainer.maybe_handle(request)
    if handled:
        return handled

    return shortcuts.render_to_response('dash_containers_create.html', {
        'create_form': form,
    }, context_instance=template.RequestContext(request))
